#!/usr/bin/R

##############################################################################
#
# MODULE:       r.avaflow.hydrograph.r
# AUTHOR:       Martin Mergili
#
# PURPOSE:      The simulation model for avalanche and debris flows
#               Script for the creation of hydrograph plots
#
# COPYRIGHT:    (c) 2013 - 2015 by the author
#               (c) 2013 - 2015 by the BOKU University, Vienna
#               (c) 2015 - 2016 by the University of Vienna
#               (c) 1993 - 2015 by the R Development Core Team
#
# VERSION:      20160813 (13 August 2016)
#
#               This program is free software under the GNU General Public
#               License (>=v2).
#
##############################################################################

#Loading library
library(stats)

#Importing arguments defined in r.avaflow.py
args<-commandArgs(trailingOnly=TRUE)
prefix<-args[1] #prefix for file names
model<-args[2] #type of model
nhyd<-as.integer(args[3]) #number of hydrograph
nhydin<-as.integer(args[4]) #total number of input hydrographs
tint<-as.numeric(args[5]) #total number of input hydrographs

#Defining data file name
hydtable=paste(prefix,'_results/',prefix,'_files/',prefix,'_hydinfo',as.character(nhyd),'.txt',sep='')

#Creating vectors from data
thyd<-read.table(hydtable, skip=1)[,1] #time passed
hhyd<-read.table(hydtable, skip=1)[,2] #(solid) flow height
vhyd<-read.table(hydtable, skip=1)[,3] #(solid) velocity
rhyd<-read.table(hydtable, skip=1)[,4] #pressure or energy
ehyd<-read.table(hydtable, skip=1)[,5] #depth of (solid) entrainment/deposition
qhyd<-read.table(hydtable, skip=1)[,6] #(solid) discharge

tmax<-round(max(thyd, na.rm=TRUE),0) #maximum value of time passed
hmax<-max(hhyd, na.rm=TRUE) #maximum value of (solid) flow height
vmax<-max(abs(vhyd), na.rm=TRUE) #maximum value of (solid) velocity
rmax<-max(rhyd, na.rm=TRUE) #maximum value of pressure or energy
emax<-max(ehyd, na.rm=TRUE) #maximum value of depth of (solid) entrainment/deposition
qmax<-max(abs(qhyd), na.rm=TRUE) #maximum value of (solid) discharge

if (model == '2' ) {

  hhyd2<-read.table(hydtable, skip=1)[,7] #fluid flow height
  vhyd2<-read.table(hydtable, skip=1)[,8] #fluid velocity
  ehyd2<-read.table(hydtable, skip=1)[,9] #depth of fluid entrainment/deposition
  qhyd2<-read.table(hydtable, skip=1)[,10] #fluid discharge

  hmax2<-max(hhyd2, na.rm=TRUE) #maximum value of fluid flow height
  vmax2<-max(abs(vhyd2), na.rm=TRUE) #maximum value of fluid velocity
  emax2<-max(ehyd2, na.rm=TRUE) #maximum value of depth of fluid entrainment/deposition
  qmax2<-max(abs(qhyd2), na.rm=TRUE) #maximum value of fluid discharge
} else {
  hmax2<-0
  qmax2<-0
}

#Defining labels and scaling
ltext=vector('expression', 8) #initializing label vector

if ( nhyd <= nhydin ) {
  textcol='green'
  ltext[1]=substitute(expression(Input~hydrograph~IH~fnhyd), list(fnhyd = format(nhyd)))[2] #title text for input hydrograph
} else {
  textcol='purple'
  ltext[1]=substitute(expression(Output~hydrograph~OH~fnhyd), list(fnhyd = format(nhyd-nhydin)))[2] #title text for output hydrograph
}

ltext[2]=substitute(expression(Time~passed~italic(T)~(s)))[2] #solid discharge
ltext[3]=substitute(expression(Flow~height~italic(H)~(m)))[2] #fluid flow height

if ( max(qmax,qmax2)<1000 ) { #right y axis label:
  mconv='1'
  ltext[4]=substitute(expression(Discharge~italic(Q)~(m^3~s^-1)))[2]
} else if ( max(qmax,qmax2)<1000000 ) {
  mconv='0.001'
  ltext[4]=substitute(expression(Discharge~italic(Q)~(10^3~m^3~s^-1)))[2]
} else {
  mconv='0.000001'
  ltext[4]=substitute(expression(Discharge~italic(Q)~(10^6~m^3~s^-1)))[2]
}
qmax<-qmax*as.numeric(mconv) #scaling of right y axis
qmax2<-qmax2*as.numeric(mconv)

#Creating plot file
hydplot=paste(prefix, '_results/' ,prefix,'_plots/' ,prefix,'_hydrograph',nhyd,'.png', sep='')
png(filename=hydplot, width=15, height=10, units='cm', res=300)

#Defining external margins
par(mar=c(3.2,3.2,1.0,3.2))

#Defining internal margins
if ( hmax > 2*hmax2 && qmax > 2*qmax2 ) {
  marg<-1.1
  tpos<--hmax*0.85
  lpos<-'bottomright'
} else if ( hmax2 > 2*hmax && qmax2 > 2*qmax ) {
  marg<-1.1
  tpos<-hmax2*0.85
  lpos<-'topright'
} else {
  marg<-1.3
  lpos<-'bottomright'
  tpos<-max(hmax,hmax2)*1.25
}

#Plotting bars
if ( model == '1' ) {
  barplot(qhyd*as.numeric(mconv), axes=F, xlab=NA, ylab=NA, ylim=c(-qmax*marg,qmax*marg), border=NA, col=rgb(0.5,0.85,0.5)) #discharge
} else if ( model == '2' ) {
  barplot(qhyd*as.numeric(mconv), axes=F, xlab=NA, ylab=NA, ylim=c(-max(qmax2*marg,qmax*marg),max(qmax2*marg,qmax*marg)),
  border=NA, col=rgb(0.45,0.3,0.0,0.5)) #solid discharge
}

par(new=TRUE)
if ( model == '2' ) {
  barplot(-qhyd2*as.numeric(mconv), axes=F, xlab=NA, ylab=NA, ylim=c(-max(qmax2*marg,qmax*marg),max(qmax2*marg,qmax*marg)),
  border=NA, col=rgb(0.0,0.0,0.6,0.5)) #fluid discharge
}
axis(side=4, tck=-0.02, labels=NA) #y axis for velocity and random kinetic energy
axis(side=4, lwd=0, line=-0.4)

#Plotting lines
par(new=TRUE)
if ( model == '1' ) {
  plot(x=thyd, y=hhyd, lty=1, lwd=2, col=rgb(0.45,0.3,0.0), axes=F, xlab=NA, ylab=NA, type='l', ylim=c(-hmax*marg,hmax*marg)) #flow height
} else if ( model == '2' ) {
  plot(x=thyd, y=hhyd, lty=1, lwd=2, col=rgb(0.45,0.3,0.0), axes=F, xlab=NA, ylab=NA, type='l',
    ylim=c(-max(hmax*marg,hmax2*marg),max(hmax*marg,hmax2*marg)), xlim=c(-tint/2,tmax+tint/2)) #solid flow height
  lines(x=thyd, y=-hhyd2, lty=1, lwd=2, col=rgb(0.0,0.0,0.8), xlim=c(-tint/2,tmax+tint/2)) #fluid flow height
}
abline(0, 0, col='black', lwd=1) #zero line

#Text and legends
text(x=tmax/2, y=tpos, labels=ltext[1], font=2, col=textcol, cex=1.1) #title text

if ( model == '1' ) {

  ltext[5]=substitute(expression(italic(H)))[2] #flow height
  ltext[6]=substitute(expression(italic(Q)))[2] #discharge

  legend(lpos, legend=ltext[5], lty=1, lwd=2, col='brown', text.col='black', bty='n', inset=c(1.025,-0.025), horiz=TRUE)
      #flow height
  legend(lpos, legend=ltext[6], border=NA, fill=rgb(0.5,0.85,0.5), text.col='black', bty='n', inset=c(0.225,-0.025), horiz=TRUE)
      #discharge

} else if ( model == '2' ) {

  ltext[5]=substitute(expression(italic(H)[s]))[2] #solid flow height
  ltext[6]=substitute(expression(italic(Q)[s]))[2] #solid discharge
  ltext[7]=substitute(expression(italic(-H)[f]))[2] #fluid flow height
  ltext[8]=substitute(expression(italic(-Q)[f]))[2] #fluid discharge

  legend(lpos, legend=ltext[5], lty=1, lwd=2, col=rgb(0.45,0.3,0.0), text.col='black', bty='n', inset=c(0.825,-0.025), horiz=TRUE)
      #solid flow height
  legend(lpos, legend=ltext[7], lty=1, lwd=2, col=rgb(0.0,0.0,0.8), text.col='black', bty='n', inset=c(0.625,-0.025), horiz=TRUE)
      # fluid flow height
  legend(lpos, legend=ltext[6], border=NA, fill=rgb(0.45,0.3,0.0,0.5), text.col='black', bty='n', inset=c(0.225,-0.025), horiz=TRUE)
      #solid discharge
  legend(lpos, legend=ltext[8], border=NA, fill=rgb(0.0,0.0,0.6,0.5), text.col='black', bty='n', inset=c(0.025,-0.025), horiz=TRUE)
      #fluid discharge
}

#Plotting bounding box and axes
box()
axis(side=1, tck=-0.02, labels=NA, at=c(0,tmax/10,2*tmax/10,3*tmax/10,4*tmax/10,5*tmax/10,6*tmax/10,7*tmax/10,8*tmax/10,9*tmax/10,tmax))
  #x axis
axis(side=1, lwd=0, line=-0.4, at=c(0,tmax/10,2*tmax/10,3*tmax/10,4*tmax/10,5*tmax/10,6*tmax/10,7*tmax/10,8*tmax/10,9*tmax/10,tmax))
axis(side=2, tck=-0.02, labels=NA) #y axis for lines
axis(side=2, lwd=0, line=-0.4)

#Plotting axis labels
mtext(side=1, ltext[2], line=2) #x axis
mtext(side=2, ltext[3], line=2) #y axis for lines
if ( model == '1' ) {
  mtext(side=4, ltext[4], line=2) #y axis for bars
} else if ( model == '2' ) {
  mtext(side=4, ltext[4], line=2) #y axis for bars
}

#Closing plot file
dev.off()
