#!/usr/bin/python

##############################################################################
#
# MODULE:       r.avaflow.py
#
# AUTHOR:       Martin Mergili
# CONTRIBUTORS: Massimiliano Alvioli, Jan-Thomas Fischer, Ivan Marchesini
#
# PURPOSE:      The simulation model for avalanche and debris flows
#               Input, data management and output script
#
# COPYRIGHT:    (c) 2013 - 2016 by the author
#               (c) 2013 - 2016 by the BOKU University, Vienna
#               (c) 2015 - 2016 by the University of Vienna
#               (c) 2000 - 2016 by the GRASS Development Team
#
# VERSION:      20160817 (17 August 2016)
#
#               This program is free software under the GNU General Public
#               License (>=v2). Read the file COPYING that comes with GRASS
#               for details.
#
##############################################################################

#%module
#% description: The simulation model for granular avalanches and debris flows
#% keywords: Raster
#% keywords: Granular avalanche
#% keywords: Debris flow
#%end

#%flag
#% key: e
#% description: Model execution
#% guisection: flags
#%end

#%flag
#% key: k
#% description: Keep result GRASS raster maps
#% guisection: flags
#%end

#%flag
#% key: m
#% description: Multiple model runs
#% guisection: flags
#%end

#%flag
#% key: v
#% description: Validation and visualization
#% guisection: flags
#%end

#%option
#% key: prefix
#% type: string
#% description: Prefix for ouput files and folders
#% required: yes
#% multiple: no
#%end

#%option
#% key: cores
#% type: string
#% description: Number of cores to be used for multiple model runs (-m)
#% required: no
#% multiple: no
#%end

#%option
#% key: cellsize
#% type: string
#% description: Pixel size in metres (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: model
#% type: string
#% description: Type of model (1 = Voellmy-Salm one-phase, 2 = Pudasaini two-phase) (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: topography
#% type: string
#% description: Type of topography (1 = slope, 2 = channel, -e)
#% required: no
#% multiple: no
#%end

#%option
#% key: funcctrl
#% type: string
#% description: Function controls (comma-separated, -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: aoicoords
#% type: string
#% description: Set of coordinates delineating area of interest (N,S,E,W)
#% required: no
#% multiple: yes
#%end

#%option
#% key: aoimap
#% type: string
#% gisprompt: old,raster,cell
#% description: Name of area of interest raster map (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: elevation
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of elevation raster map (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: hrelease
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of release height raster map (comma-separated, -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: rhreleases
#% type: string
#% description: Ratio(s) of solid release height (comma-separated, -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: vhrelease
#% type: string
#% description: Variation of release height (comma-separated, -m)
#% required: no
#% multiple: yes
#%end

#%option
#% key: hreleases
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of solid release height raster map (-e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: hreleasef
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of fluid release height raster map (-e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: hentrmax
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of maximum height of entrainment raster map (comma-separated, -e)
#% required: no
#% multiple: no
#%end

#%option
#% key: rhentrmaxs
#% type: string
#% description: Ratio(s) of height of solid entrainment (comma-separated, -e)
#% required: no
#% multiple: no
#%end

#%option
#% key: vhentrmax
#% type: string
#% description: Variation of maximum height of entrainment (comma-separated, -m)
#% required: no
#% multiple: yes
#%end

#%option
#% key: hentrmaxs
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of maximum height of solid entrainment raster map (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: hentrmaxf
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of maximum height of fluid entrainment raster map (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: centr
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of entrainment coefficient raster map (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: delta
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of basal friction angle raster map (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: impactarea
#% type: string
#% gisprompt: old,raster,cell
#% description: Name of observed impact area raster map (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: hdeposit
#% type: string
#% gisprompt: old,raster,dcell
#% description: Name of observed deposit height raster map (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: hydrograph
#% type: string
#% description: Path(es) to hydrograph file(s) (-e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: hydrocoords
#% type: string
#% description: Pairs of coordinates of hydrograph(s) (x1, y1, x2, y2, ..., -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: flowparam
#% type: string
#% description: Flow parameters (comma-separated, -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: baseparam
#% type: string
#% description: Basal surface parameters (comma-separated, -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: sampling
#% type: string
#% description: Type of parameter sampling (positive number = random, 0 = controlled, negative number = OAT, -m)
#% required: no
#% multiple: no
#%end

#%option
#% key: cfl
#% type: string
#% description: CFL criterion, maximum time step length (s) (-e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: tint
#% type: string
#% description: Time interval for writing output (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: tstop
#% type: string
#% description: Time after which to stop simulation (-e)
#% required: no
#% multiple: no
#%end

#%option
#% key: imparam
#% type: string
#% description: Lower thresholds for display of flow height, kinetic energy and pressure (comma-separated, -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: profile
#% type: string
#% description: Pairs of coordinates for profile plots (x1, y1, x2, y2, ..., -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: ctrlpoints
#% type: string
#% description: Pairs of coordinates for control points (x1, y1, x2, y2, ..., -e)
#% required: no
#% multiple: yes
#%end

#%option
#% key: phexagg
#% type: string
#% description: Exaggeration of flow height in profile plots (-e)
#% required: no
#% multiple: no
#%end


# Importing libraries

import grass.script as grass
from grass.script import core as grasscore
import images2gif
from images2gif import writeGif
import math
from PIL import Image
import os
import Queue
import random
import re
import shutil
import subprocess
import sys
import threading
import time


# Defining fundamental functions, classes and variables

queueLock=threading.Lock() #queue for multi-core processing
workQueue=Queue.Queue()

exitFlag=0 #exit flag
ambvars=grass.gisenv() #path to GRASS data
locpath=ambvars.GISDBASE+'/'+ambvars.LOCATION_NAME #path to GRASS location
temppath=ambvars.GISDBASE+'/'+ambvars.LOCATION_NAME+'/'+ambvars.MAPSET+'/.tmp/rtemp' #path to temporary directory
scriptpath=os.environ['GISBASE']+'/scripts' #path to GRASS scripts
mainmapset=ambvars.MAPSET #name of main mapset

def which(file): #function for reading GRASS directory:
    for path in os.environ['PATH'].split(':'):
        if os.path.exists(path+'/'+file):
            return path+'/'+file
    return None

bingrass=which('grass70') #reading GRASS directory

def ErrorMessage(): #function for error message:
    grass.message(' ')
    grass.error('Please specify all necessary parameters.')
    grass.message(' ')
    sys.exit()

class myThread (threading.Thread): #class for threading:
    def __init__(self, name, q):
        threading.Thread.__init__(self)
        self.name=name
        self.q=q
    def run(self):
        process_data(self.name, self.q)

def StartBatch(jid): #function for start of multi-core processing:
    print 'Executing model run %s' %jid
    execute='bash '+temppath+'/tmp'+str(jid)+'/batch'+str(jid)
    os.system(execute+' < /dev/null > '+temppath+'/out'+str(jid))
    print 'Model run %s completed.' %jid
    return

def process_data(threadName, q): #function supporting start of multi-core processing:
    while not exitFlag:
        queueLock.acquire()
        if not workQueue.empty():
            time.sleep(0.25)
            data=q.get()
            queueLock.release()
            StartBatch(data)
        else:
            queueLock.release()

mstring=['_hflow','_tflow','_pflow','_basechange','_vflow'] #list of names of raster maps

def corrasc( corrname ): #function for correcting output ascii rasters

    os.rename( corrname+'.asc', corrname+'o.asc' )

    fpressureo=open(corrname+'o.asc', 'r')
    fpressure=open(corrname+'.asc', 'w')

    fdata=''
    for i in range(0,5):
        fdatah=fpressureo.readline()
        fdata=fdata+'\t'.join(fdatah.split())+'\n'

    fdatah=fpressureo.readline()
    fdatai=fdatah.split()
    if fdatai[0]=='NODATA_value':
        fdata=fdata+fdatah
        fdataj=fdatai[1]
    else:
        fdata=fdata+'NODATA_value\t-9999\n'+fdatah
        fdataj='-9999'

    fdata=fdata+''.join(fpressureo.readlines())
    fdata=fdata.replace('corner','center').replace(fdataj,'-9999')

    fpressure.write(fdata)

    fpressureo.close()
    fpressure.close()

    os.system('rm -f '+corrname+'o.asc')
    os.system('rm -f '+corrname+'.asc.aux.xml')

def main(): #starting main function


# Setting flags and parameters

    eflag=flags['e']
    kflag=flags['k']
    mflag=flags['m']
    vflag=flags['v']
    pf=options['prefix']
    cores=options['cores']
    cellsize=options['cellsize']
    model=options['model']
    topo=options['topography']
    funcctrl=options['funcctrl']
    aoicoords=options['aoicoords']
    aoi=options['aoimap']
    elevation=options['elevation']
    hrelease=options['hrelease']
    rhreleases=options['rhreleases']
    vhrelease=options['vhrelease']
    hreleases=options['hreleases']
    hreleasef=options['hreleasef']
    hentrmax=options['hentrmax']
    rhentrmaxs=options['rhentrmaxs']
    vhentrmax=options['vhentrmax']
    hentrmaxs=options['hentrmaxs']
    hentrmaxf=options['hentrmaxf']
    centr=options['centr']
    delta=options['delta']
    impactarea=options['impactarea']
    hdeposit=options['hdeposit']
    hydrograph=options['hydrograph']
    hydrocoords=options['hydrocoords']
    flowparam=options['flowparam']
    baseparam=options['baseparam']
    sampling=options['sampling']
    cfl=options['cfl']
    tint=options['tint']
    tstop=options['tstop']
    imparam=options['imparam']
    profile=options['profile']
    ctrlpoints=options['ctrlpoints']
    phexagg=options['phexagg']


# Making sure that all necessary parameters are defined

    if not pf: ErrorMessage()

    if not eflag and not vflag: #if mode was not specified, running entire program
        eflag='1'
        vflag='1'

    if eflag: #for model execution mode:

        if not model or not topo or (topo=='2' and not aoi) or not elevation or not flowparam or not tint or not tstop or not imparam:
            ErrorMessage()
        if not model=='1' and not model=='2': ErrorMessage()
        if model=='1' and (not hydrograph and not hrelease): ErrorMessage()
        elif model=='2' and (not hydrograph and ((not hrelease or not rhreleases) and (not hreleases and not hreleasef))): ErrorMessage()
        if hydrograph and not hydrocoords: ErrorMessage()
        if profile and not phexagg: phexagg='1'

        if not sampling: sampling='0'

        if model=='2':
            if ( hreleases and not hreleasef ) or ( hreleasef and not hreleases ) or ( hrelease and not rhreleases ): ErrorMessage() 
            if ( hentrmaxs and not hentrmaxf ) or ( hentrmaxf and not hentrmaxs ) or ( hentrmax and not rhentrmaxs ): ErrorMessage()

    if mflag and not cores: ErrorMessage()
        #if parameters for multiple simulations were not specified along with flag, exiting with error message


    print
    print
    print '#############################################################################'
    print '#'
    print '# MODULE:       r.avaflow'
    print '#'
    print '# AUTHORS:      Martin Mergili and Shiva P. Pudasaini'
    print '# CONTRIBUTORS: Massimiliano Alvioli, Wolfgang Fellin, Jan-Thomas Fischer,'
    print '#               Ivan Marchesini, Markus Metz, Alexander Ostermann,'
    print '#               Matthias Rauter'
    print '#'
    print '# PURPOSE:      The simulation model for avalanche and debris flows'
    print '# VERSION:      20160920 (20 September 2016)'
    print '#'
    print '# COPYRIGHT:    (c) 2013 - 2016 by the authors'
    print '#               (c) 2013 - 2016 by the BOKU University, Vienna'
    print '#               (c) 2015 - 2016 by the University of Vienna'
    print '#               (c) 2014 - 2016 by the University of Bonn'
    print '#               (c) 2000 - 2016 by the GRASS Development Team'
    print '#               (C) 1993 - 2016 by the R Development Core Team'
    print '#'
    print '#               This program is free software under the GNU General Public'
    print '#               License (>=v2). Read the file COPYING that comes with GRASS'
    print '#               for details.'
    print '#'
    print '# PREFIX:       %s' %pf
    print '#'
    print '#############################################################################'
    print
    print
    print '1. PREPARING ENVIRONMENT.'


# Preparing environment

    os.system('rm -rf '+temppath) #if necessary, deleting temporary directory
    os.system('mkdir '+temppath) #creating temporary directory


    os.environ['GRASS_VERBOSE']='-1'
    grass.run_command('g.gisenv', set='GRASS_VERBOSE=-1') #suppressing errors and warnings (overruled for some functions)

    ascpath=pf+'_results/'+pf+'_ascii/' #path to directory with ascii rasters
    filepath=pf+'_results/'+pf+'_files/' #path to directory with result text files

    if eflag: #for model execution mode:

        if os.path.exists(pf+'_results'): os.system('rm -rf '+pf+'_results') #if result directory already exists, deleting it
        os.mkdir(pf+'_results') #creating directory for results
        os.mkdir(pf+'_results/'+pf+'_files') #creating directory for result files
        os.mkdir(pf+'_results/'+pf+'_plots') #creating directory for result plots
        if not mflag: os.mkdir(pf+'_results/'+pf+'_plots/'+pf+'_maps_timesteps') #creating directory for result maps of all time steps
        if not mflag: os.mkdir(pf+'_results/'+pf+'_plots/'+pf+'_profiles_timesteps')
            #creating directory for result profiles of all time steps
        os.mkdir(ascpath) #creating directory for result ascii rasters

        if model=='1': lmax=7 #number of flow parameters
        elif model=='2': lmax=14
        if baseparam: bmax=1 #number of basal surface parameters
        else: bmax=0


    # Defining GRASS region

        grass.run_command('g.region', flags='d') #setting default region
        if cellsize: grass.run_command('g.region', flags='a', res=cellsize) #updating pixel size
        if aoicoords: #updating extent:
            aoicoords=map(str, aoicoords.split(','))
            if not len(aoicoords)==4: ErrorMessage()
            grass.run_command('g.region', flags='a', n=aoicoords[0], s=aoicoords[1], w=aoicoords[2], e=aoicoords[3])

        if aoi and topo=='2': grass.mapcalc('"_aoi"=if("%s">0,1,null())' %aoi, overwrite=True, quiet=True) #area of interest (aoi)
        else: grass.mapcalc('"_aoi"=pow("%s",0)' %elevation, overwrite=True, quiet=True)

        cellsize=grass.raster_info('_aoi')['nsres'] #reading pixel size
        rnorth=grass.raster_info('_aoi')['north'] #reading boundaries of aoi map
        rsouth=grass.raster_info('_aoi')['south']
        rwest=grass.raster_info('_aoi')['west']
        reast=grass.raster_info('_aoi')['east']

        grass.run_command('g.region', flags='d') #setting default region
        grass.run_command('g.region', flags='a', n=rnorth, s=rsouth, w=rwest, e=reast) #updating bounds
        grass.run_command('g.region', flags='a', res=cellsize) #updating pixel size


    # Splitting comma-separated parameter sets

        flowparam=map(str, flowparam.split(',')) #splitting string with flow parameters
        if baseparam:
            baseparam=map(str, baseparam.split(',')) #splitting string with basal surface parameters
        if cfl:
            cfl=map(str, cfl.split(',')) #splitting string with CFL criterion and maximum time step length
            if not len(cfl)==2: ErrorMessage()
        else: #if parameter is not provided, using default values:
            cfl=[0.2]
            cfl.append(0.2)
        imparam=map(str, imparam.split(',')) #splitting string with parameter thresholds
        if not len(imparam)==4: ErrorMessage()
        imparamc=imparam[3]

        if funcctrl:
            funcctrl=map(str, funcctrl.split(',')) #splitting string with function controls
            if not len(funcctrl)==6: ErrorMessage()
        else: #if parameter is not given, defining default values:
            funcctrl=[0]
            funcctrl.append(0)
            funcctrl.append(0)
            funcctrl.append(0)
            funcctrl.append(0)
            funcctrl.append(0)


    # Pre-processing input raster maps

        if topo=='2': #if area of interest map is needed:
            grass.run_command('g.copy', rast=(aoi,pf+'_aoi'), overwrite=True, quiet=True) #area of interest map
            grass.run_command('r.out.gdal', input=pf+'_aoi', output=ascpath+pf+'_aoi.asc', format='AAIGrid',
                overwrite=True) #exporting to ascii
            corrasc(ascpath+pf+'_aoi')

        grass.run_command('g.copy', rast=(elevation,pf+'_elev'), overwrite=True, quiet=True) #elevation map
        grass.run_command('r.out.gdal', input=pf+'_elev', output=ascpath+pf+'_elev.asc', format='AAIGrid',
            overwrite=True) #exporting to ascii
        corrasc(ascpath+pf+'_elev')

        if hrelease: grass.run_command('g.copy', rast=(hrelease,pf+'_hflow0000'), overwrite=True, quiet=True) #release height map
        else: grass.mapcalc('"%s_hflow0000"=0' %pf, overwrite=True, quiet=True)
        grass.run_command('r.out.gdal', input=pf+'_hflow0000', output=ascpath+pf+'_hflow0000.asc', format='AAIGrid',
            overwrite=True) #exporting to ascii
        corrasc(ascpath+pf+'_hflow0000')

        if model=='2' and hreleases:
            grass.run_command('g.copy', rast=(hreleases,pf+'_hflows0000'),
                overwrite=True, quiet=True) #solid release height map
            grass.run_command('r.out.gdal', input=pf+'_hflows0000', output=ascpath+pf+'_hflows0000.asc', format='AAIGrid',
                overwrite=True) #exporting to ascii
            corrasc(ascpath+pf+'_hflows0000')

        if model=='2' and hreleasef:
            grass.run_command('g.copy', rast=(hreleasef,pf+'_hflowf0000'),
                overwrite=True, quiet=True) #fluid release height map
            grass.run_command('r.out.gdal', input=pf+'_hflowf0000', output=ascpath+pf+'_hflowf0000.asc', format='AAIGrid',
                overwrite=True) #exporting to ascii
            corrasc(ascpath+pf+'_hflowf0000')

        if hentrmax:
            grass.run_command('g.copy', rast=(hentrmax,pf+'_hentrmax'), overwrite=True, quiet=True) #maximum height of entrainment map
            grass.run_command('r.out.gdal', input=pf+'_hentrmax', output=ascpath+pf+'_hentrmax.asc', format='AAIGrid',
                overwrite=True) #exporting to ascii
            corrasc(ascpath+pf+'_hentrmax')

        if model=='2' and hentrmaxs:
            grass.run_command('g.copy', rast=(hentrmaxs,pf+'_hentrmaxs'),
            overwrite=True, quiet=True) #maximum height of solid entrainment map
            grass.run_command('r.out.gdal', input=pf+'_hentrmaxs', output=ascpath+pf+'_hentrmaxs.asc', format='AAIGrid',
                overwrite=True) #exporting to ascii
            corrasc(ascpath+pf+'_hentrmaxs')

        if model=='2' and hentrmaxf:
            grass.run_command('g.copy', rast=(hentrmaxf,pf+'_hentrmaxf'),
            overwrite=True, quiet=True) #maximum height of fluid entrainment map
            grass.run_command('r.out.gdal', input=pf+'_hentrmaxf', output=ascpath+pf+'_hentrmaxf.asc', format='AAIGrid',
                overwrite=True) #exporting to ascii
            corrasc(ascpath+pf+'_hentrmaxf')

        if impactarea:
            grass.run_command('g.copy', rast=(impactarea,pf+'_impactarea'), overwrite=True, quiet=True) #observed impact area map

            maximpactarea=grass.raster_info(impactarea)['max'] #maximum value of impact area
            if maximpactarea>0: impdef='1' #setting control for impact area map
            else: impdef='0'

            grass.run_command('r.out.gdal', input=pf+'_impactarea',
                output=ascpath+pf+'_impactarea.asc', format='AAIGrid', overwrite=True) #exporting to ascii
            corrasc(ascpath+pf+'_impactarea')
        else: impdef='0' #setting control for impact area map to negative

        if hdeposit:
            grass.run_command('g.copy', rast=(hdeposit,pf+'_hdeposit'), overwrite=True, quiet=True) #observed height of deposit map

            maxhdeposit=grass.raster_info(hdeposit)['max'] #maximum value of height of deposit
            if maxhdeposit>0: depdef='1' #setting control for deposit map
            else: depdef='0'

            grass.run_command('r.out.gdal', input=pf+'_hdeposit', output=ascpath+pf+'_hdeposit.asc', format='AAIGrid',
                overwrite=True) #exporting to ascii
            corrasc(ascpath+pf+'_hdeposit')
        else: depdef='0' #setting control for height of deposit map to negative

        if model=='2' and hrelease and rhreleases: #if release heights are defined by total height and ratio of solid:

            hreleases=pf+'_hflows0000'
            hreleasef=pf+'_hflowf0000'

            if mflag==0: #for single model run:

                grass.mapcalc('"%s"="%s"*%s' %(hreleases,hrelease.replace('.asc',''),rhreleases),
                    overwrite=True, quiet=True) #solid release height map
                grass.mapcalc('"%s"="%s"*(1-%s)' %(hreleasef,hrelease.replace('.asc',''),rhreleases),
                    overwrite=True, quiet=True) #fluid release height map

                grass.run_command('r.out.gdal', input=hreleases.replace('.asc',''), output=ascpath+hreleases+'.asc', format='AAIGrid',
                    overwrite=True) #exporting solid release height map to ascii
                grass.run_command('r.out.gdal', input=hreleasef.replace('.asc',''), output=ascpath+hreleasef+'.asc', format='AAIGrid',
                    overwrite=True) #exporting fluid release height map to ascii

                corrasc(ascpath+hreleases)
                corrasc(ascpath+hreleasef)

        if model=='2' and hentrmax and rhentrmaxs: #if maximum heights of entrainment are defined by total height and ratio of solid:

            hentrmaxs=pf+'_hentrmaxs'
            hentrmaxf=pf+'_hentrmaxf'

            if mflag==0: #for single model run:
                grass.mapcalc('"%s"="%s"*%s' %(hentrmaxs,hentrmax.replace('.asc',''),rhentrmaxs),
                    overwrite=True, quiet=True) #maximum solid height of entrainment map
                grass.mapcalc('"%s"="%s"*(1-%s)' %(hentrmaxf,hentrmax.replace('.asc',''),rhentrmaxs),
                    overwrite=True, quiet=True) #maximum fluid height of entrainment map

                grass.run_command('r.out.gdal', input=hentrmaxs.replace('.asc',''), output=ascpath+hentrmaxs+'.asc', format='AAIGrid',
                    overwrite=True) #exporting maximum height of solid entrainment map to ascii
                grass.run_command('r.out.gdal', input=hentrmaxf.replace('.asc',''), output=ascpath+hentrmaxf+'.asc', format='AAIGrid',
                    overwrite=True) #exporting maximum height of fluid entrainment map to ascii

                corrasc(ascpath+hentrmaxs)
                corrasc(ascpath+hentrmaxf)

        if model=='1': #controls for release masses and entrainment:

            if hrelease: releasemass=1
            else: releasemass=0

            if hentrmax: entrainment=1
            else: entrainment=0

        elif model=='2':

            if hreleases and hreleasef: releasemass=1
            else: releasemass=0

            if hentrmaxs and hentrmaxf: entrainment=1
            else: entrainment=0

        if not funcctrl[4]=='0' or not funcctrl[5]=='0': basechange=1
        else: basechange=0


    # Pre-processing hydrograph information

        if hydrograph: #if hydrograph file is defined:

            hydrograph=map(str, hydrograph.split(',')) #splitting list of hydrographs
            hyd_nin=len(hydrograph)

            hydlines=[]
            for hydi in range(0,hyd_nin):
                fhyd=open(hydrograph[hydi], 'r') #opening hydrograph file
                hydlines.append(sum(1 for line in fhyd)-2) #counting number of lines
                fhyd.close() #closing file
        else: hyd_nin=0

        if hydrocoords: #if hydrograph coordinates are defined:

            hydrocoords=map(str, hydrocoords.split(',')) #splitting hydrograph coordinate input
            hyd_nout=len(hydrocoords)/4-hyd_nin


    # Pre-processing control point information

        if ctrlpoints: #if control point coordinates are defined:

            ctrlpoints=map(str, ctrlpoints.split(',')) #splitting list of control point coordinates
            lctrlpts=len(ctrlpoints)/2 #number of control points

            xctrl=[]
            yctrl=[]
            for ictrlpts in range(0,lctrlpts): #extracting control point coordinates:
                xctrl.append(ctrlpoints[2*ictrlpts]) 
                yctrl.append(ctrlpoints[2*ictrlpts+1])
        else: lctrlpts=0

        if not mflag: #for single model run:


        # Writing model parameters to file

            p1file=open(temppath+'/param1.txt', 'w') #opening parameter files

            gt=[float(flowparam[0])]
            for l in range(1,lmax): gt.append(float(flowparam[l]))

            if baseparam:
                bp=[baseparam[0]]
                for l in range(1,bmax): bp.append(baseparam[l])

            print>> p1file, mainmapset #name of main mapset
            print>> p1file, '0' #identifier for single model run
            print>> p1file, model #model
            print>> p1file, topo #topography

            print>> p1file, funcctrl[0] #control for correction of flow height
            print>> p1file, funcctrl[1] #control for diffusion control
            print>> p1file, funcctrl[2] #control for conservation of volume
            print>> p1file, funcctrl[3] #control for surface control
            print>> p1file, funcctrl[4] #control for entrainment and deposition
            print>> p1file, funcctrl[5] #control for suppression of stopping

            if baseparam and (not funcctrl[4]=='0'):
                if not bmax==1: ErrorMessage()
                #making sure that basal surface parameters are provided correctly

            print>> p1file, str(releasemass) #control for solid release height
            if model=='2': print>> p1file, str(releasemass) #control for (fluid) release height
            print>> p1file, str(entrainment) #control for maximum height of (solid) entrainment
            if model=='2': print>> p1file, str(entrainment) #control for maximum height of fluid entrainment
            if funcctrl[4]=='1' and centr: print>> p1file, '1' #control for entrainment coefficient map
            else: print>> p1file, '0' 
            if delta: print>> p1file, '1' #control for basal friction angle map
            else: print>> p1file, '0' 
            if hydrograph or hydrocoords: print>> p1file, '1' #control for hydrograph
            else: print>> p1file, '0' 
            if topo=='2': print>> p1file, aoi #name of area of interest map
            print>> p1file, elevation #name of elevation map
            if model=='1' and releasemass==1: print>> p1file, hrelease #name of release height map
            elif model=='2' and releasemass==1: print>> p1file, hreleases #name of solid release height map
            if model=='2' and releasemass==1: print>> p1file, hreleasef #name of fluid release height map
            if model=='1' and entrainment==1: print>> p1file, hentrmax #name of maximum height of entrainment map
            elif model=='2' and entrainment==1: print>> p1file, hentrmaxs #name of maximum height of solid entrainment map
            if model=='2' and entrainment==1: print>> p1file, hentrmaxf #name of maximum height of fluid entrainment map
            if funcctrl[4]=='1' and centr: print>> p1file, centr #name of entrainment coefficient map
            if delta: print>> p1file, delta #name of basal friction angle map

            if hydrograph or hydrocoords:
                print>> p1file, hyd_nin #number of input hydrographs
                print>> p1file, hyd_nout #number of output hydrographs
                for hydi in range(0,hyd_nin):
                    print>> p1file, hydrograph[hydi] #path to hydrograph file
                    print>> p1file, str(hydlines[hydi]) #number of hydrograph time steps 
                for hydi in range(0,hyd_nin+hyd_nout):
                    print>> p1file, hydrocoords[4*hydi] #x coordinate of hydrograph
                    print>> p1file, hydrocoords[4*hydi+1] #y coordinate of hydrograph
                    print>> p1file, hydrocoords[4*hydi+2] #length of hydrograph profile
                    print>> p1file, hydrocoords[4*hydi+3] #aspect of hydrograph profile
            else:
                hyd_nin=0
                hyd_nout=0

            print>> p1file, lmax #number of flow parameters
            for l in range(0,lmax): print>> p1file, round(gt[l],10) #flow parameters
            if baseparam and funcctrl[4]!='0':
                for l in range(0,bmax): print>> p1file, bp[l] #basal surface parameters
            elif not baseparam and funcctrl[4]!='0':
                for l in range(0,1): print>> p1file, '-9999'

            print>> p1file, imparamc #threshold of flow height (for computation)
            print>> p1file, imparam[0] #threshold of flow height (for display)
            print>> p1file, imparam[1] #threshold of flow kinetic energy
            print>> p1file, imparam[2] #threshold of flow pressure
            print>> p1file, tint #time for writing output
            print>> p1file, tstop #process duration at which to stop
            print>> p1file, cfl[0] #cfl criterion
            print>> p1file, cfl[1] #maximum length of time step

            print>> p1file, '%s_' %pf #prefix
            print>> p1file, '%s_results/%s_ascii/' %(pf,pf) #path and prefix for storing output maps
            print>> p1file, '%s_results/%s_files/' %(pf,pf) #path and prefix for storing output files

            p1file.close() #closing parameter file

            shutil.copyfile(temppath+'/param1.txt',
                pf+'_results/'+pf+'_files/'+pf+'_param.txt') #copying parameter file to results folder


        # Routing flow

            print '2. ROUTING FLOW.'

            start = time.time() #storing time (start of main computation)

            os.environ['XINT']='1' #exporting id of model run
            grass.run_command('r.avaflow.main', flags='1') #executing r.avaflow


        # Finalizing computation

            ftimesteps=open(pf+'_results/'+pf+'_files/'+pf+'_nout1.txt', 'r') #opening file with number of time steps and success
            ntimesteps=ftimesteps.readline() #reading number of time steps
            ntimesteps=int(ntimesteps.replace('\n', '')) #removing newline
            csuccess=ftimesteps.readline() #reading control for success of simulation
            csuccess=int(csuccess.replace('\n', '')) #removing newline
            vol_entr=ftimesteps.readline() #reading entrained (solid) volume
            vol_entr=float(vol_entr.replace('\n', '')) #removing newline
            vol_entr2=ftimesteps.readline() #reading entrained fluid volume
            vol_entr2=float(vol_entr2.replace('\n', '')) #removing newline
            ftimesteps.close() #closing file with number of time steps

            for j in range(0,5): #loop over impact parameters:

                if model=='1':

                    if j>0 or not hrelease: grass.mapcalc('"%s%s0000"=0' %(pf, mstring[j]), overwrite=True, quiet=True) #map
                    grass.run_command('r.out.gdal', input=pf+mstring[j]+'0000', output=ascpath+pf+mstring[j]+'0000.asc',
                        format='AAIGrid', overwrite=True) #exporting map to ascii
                    corrasc(ascpath+pf+mstring[j]+'0000')

                elif model=='2':

                    if j>0 or (not hreleases and not hrelease): grass.mapcalc('"%s%ss0000"=0' %(pf, mstring[j]),
                        overwrite=True, quiet=True) #solid map
                    grass.run_command('r.out.gdal', input=pf+mstring[j]+'s0000', output=ascpath+pf+mstring[j]+'s0000.asc',
                        format='AAIGrid', overwrite=True) #exporting solid map to ascii

                    if j>0 or (not hreleasef and not hrelease): grass.mapcalc('"%s%sf0000"=0' %(pf, mstring[j]),
                        overwrite=True, quiet=True) #fluid map
                    grass.run_command('r.out.gdal', input=pf+mstring[j]+'f0000', output=ascpath+pf+mstring[j]+'f0000.asc',
                        format='AAIGrid', overwrite=True) #exporting fluid map to ascii

                    grass.mapcalc('"%s%s0000"="%s%sf0000"+"%s%ss0000"' %(pf, mstring[j], pf, mstring[j], pf, mstring[j]),
                        overwrite=True, quiet=True) #total map
                    grass.run_command('r.out.gdal', input=pf+mstring[j]+'0000', output=ascpath+pf+mstring[j]+'0000.asc',
                        format='AAIGrid',overwrite=True) #exporting total map to ascii

                    corrasc(ascpath+pf+mstring[j]+'s0000')
                    corrasc(ascpath+pf+mstring[j]+'f0000')
                    corrasc(ascpath+pf+mstring[j]+'0000')

            if basechange==1: grass.mapcalc('"%s_hflow_dep"=if("%s_basechange_fin">0,"%s_basechange_fin",0)'
                %(pf, pf, pf), overwrite=True, quiet=True)
            else: grass.mapcalc('"%s_hflow_dep"="%s_hflow_fin"' %(pf, pf), overwrite=True, quiet=True) #simulated height of deposition map

            if model=='1': grass.mapcalc('"%s_elev_mod"="%s_elev"+"%s_basechange_fin"' %(pf, pf, pf),
                overwrite=True, quiet=True)

            elif model=='2': grass.mapcalc(
                '"%s_elev_mod"="%s_elev"+"%s_basechanges_fin"+"%s_basechangef_fin"'
                %(pf, pf, pf, pf), overwrite=True, quiet=True) #updated elevation map

            grass.run_command('r.out.gdal', input=pf+'_elev_mod', output=ascpath+pf+'_elev_mod.asc',
                format='AAIGrid', overwrite=True) #exporting updated elevation map to ascii
            corrasc(ascpath+pf+'_elev_mod')


        # Evaluating result

            if profile or impdef=='1' or depdef=='1' or not funcctrl[4]=='0':

                fval=open(pf+'_results/'+pf+'_files/'+pf+'_validation.txt', 'w') #opening file for validation parameters for writing
                print>> fval, 'Criterion\tShortcut\tValue' #writing file header

            if model=='1' and not funcctrl[4]=='0':

                print>> fval, 'Entrained volume\tVE\t'+str(vol_entr)+' cubic metres'
                print>> fval, ' '

            elif model=='2' and not funcctrl[4]=='0':

                print>> fval, 'Entrained solid volume\tVEs\t'+str(vol_entr)+' cubic metres'
                print>> fval, 'Entrained fluid volume\tVEf\t'+str(vol_entr2)+' cubic metres'
                print>> fval, ' '

            if profile: #if profile is defined:

                flowline_start=0.0 #setting initial value for start to start point of profile
                if releasemass==1: #if release height is defined:

                    grass.run_command('r.profile', input=pf+'_hflow0000',
                        output=temppath+'/fhrelease.txt', coordinates=profile, null='-9999', quiet=True, overwrite=True)
                        #profile of release height

                    fprof=open(temppath+'/fhrelease.txt', 'r') #opening profile of release height for reading
                    flines = sum(1 for line in fprof) #counting number of lines
                    fprof.close() #closing file

                    fprof=open(temppath+'/fhrelease.txt') #again, opening profile file for reading
                    for j in range(1,flines): #loop over all lines:
                        fline=fprof.readline() #reading and splitting line
                        fline=map(str, fline.split(' '))
                        if float(fline[2].replace('\n',''))>0:
                            flowline_start=float(fline[1]) #identifying profile position at start of flow
                            break
                    fprof.close() #closing file

                if impdef=='1': #identifying profile position at terminus of observed flow:

                    mstringj=0 #initializing counter
                    grass.run_command('r.profile', input=pf+'_impactarea', output=temppath+'/fimp.txt',
                        coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of map

                    flowline_stopimp = None
                    fprof=open(temppath+'/fimp.txt') #opening profile for reading
                    for j in range(1,flines): #loop over all lines:
                        fline=fprof.readline() #reading and splitting line
                        fline=map(str, fline.split(' '))
                        if float(fline[2].replace('\n',''))==1 or (not flowline_stopimp and j==flines-1):
                            flowline_stopimp=float(fline[1]) #identifying profile position at terminus of observed flow
                    fprof.close() #closing file
                    maxlength=flowline_stopimp-flowline_start #flow length
                    print>> fval, 'Flow length to terminus of observed impact area\tLio\t%s m' %maxlength

                if depdef=='1': #identifying profile position at terminus of observed flow:

                    mstringj=0 #initializing counter
                    grass.run_command('r.profile', input=pf+'_hdeposit', output=temppath+'/fdep.txt',
                        coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of map

                    flowline_stop = None
                    fprof=open(temppath+'/fdep.txt') #opening profile for reading
                    for j in range(1,flines): #loop over all lines:
                        fline=fprof.readline() #reading and splitting line
                        fline=map(str, fline.split(' '))
                        if float(fline[2].replace('\n',''))>=float(imparam[0]) or (not flowline_stop and j==flines-1):
                            flowline_stopdep=float(fline[1]) #identifying profile position at terminus of observed flow
                    fprof.close() #closing file
                    maxlength=flowline_stopdep-flowline_start #flow length
                    print>> fval, 'Flow length to terminus of observed deposit\tLdo\t%s m' %maxlength

                mstringj=0 #initializing counter
                for mstringi in ['_hflow_max','_hflow_dep']:

                    grass.run_command('r.profile', input=pf+mstringi, output=temppath+'/f'+mstringi+'.txt',
                        coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of map

                    flowline_stop = None
                    fprof=open(temppath+'/f'+mstringi+'.txt') #opening profile for reading
                    for j in range(1,flines): #loop over all lines:
                        fline=fprof.readline() #reading and splitting line
                        fline=map(str, fline.split(' '))
                        if float(fline[2].replace('\n',''))>=float(imparam[mstringj]) or (not flowline_stop and j==flines-1):
                            flowline_stop=float(fline[1]) #identifying profile position at termination of flow
                    fprof.close() #closing file
                    maxlength=flowline_stop-flowline_start #flow length
                    if mstringi=='_hflow_dep': print>> fval, 'Flow length to terminus of modelled deposit\tLdm\t%s m' %maxlength
                    elif mstringi=='_hflow_max': print>> fval, 'Flow length to terminus of modelled impact area\tLim\t%s m' %maxlength
                        #writing flow length to text file
                    if impdef=='1' and mstringi=='_hflow_max':
                        flowline_diff=flowline_stop-flowline_stopimp
                        print>> fval, 'Difference between modelled and observed flow length (impact area)\tdLi\t%s m' %str(flowline_diff)
                    elif depdef=='1' and mstringi=='_hflow_dep': #differences between modelled and observed flow lengths:
                        flowline_diff=flowline_stop-flowline_stopdep
                        print>> fval, 'Difference between modelled and observed flow length (deposit)\tdLd\t%s m' %str(flowline_diff)

                    if not mstringi=='_hflow_max': mstringj=mstringj+1 #updating counter

                print>> fval, ''

            if impdef=='1' and depdef=='1': mstringlist=['_hflow_max','_hflow_dep'] #raster maps to be processed
            elif impdef=='1': mstringlist=['_hflow_max']
            elif depdef=='1': mstringlist=['_hflow_dep']
            else: mstringlist=[]

            mstringj=0
            for mstringi in mstringlist: #loop over all maps to be processed:

                if mstringi=='_hflow_max':
                    mstringref='_impactarea'
                    grass.mapcalc('"_obsbin"=if("%s%s">0,1,if("%s%s"==0,0,null()))' %(pf,mstringref,pf,mstringref),
                        overwrite=True, quiet=True) #processed map of observed impact area

                elif mstringi=='_hflow_dep':
                    mstringref='_hdeposit' 
                    grass.mapcalc('"_obsbin"=if("%s%s">=%s,1,if("%s%s"==0,0,null()))' %(pf,mstringref,imparam[mstringj],pf,mstringref),
                        overwrite=True, quiet=True) #processed map height of observed deposit

                grass.mapcalc('"_simbin"=if("%s%s">=%s,1,0)' %(pf,mstringi,imparam[mstringj]), overwrite=True, quiet=True)
                    #processed map of simulated impact area or deposit
                grass.mapcalc('"_eval"="_simbin"+10*"_obsbin"', overwrite=True, quiet=True) #combination of observed and simulated deposits
                grass.run_command('r.stats', flags='c'+'n', separator='x', input='_eval', output='%s/validation.txt'
                    %temppath, quiet=True, overwrite=True) #scores tp=11, tn=0, fp=1, fn=10

                valfile=open(temppath+'/validation.txt', 'r') #opening raw file with validation scores
                valfile_lines=sum(1 for line in valfile) #reading number of lines
                valfile.close() #closing file

                if valfile_lines<4: #if number of lines is smaller than 4 (at least one of the scores is 0):
                    valfile=open(temppath+'/validation.txt', 'r') #opening file again
                    lr=valfile.readline() #reading first line
                    l1=lr.find('0x') #checking for tn score
                    if l1>-1: lr=valfile.readline() #if found, reading next line
                    l2=lr.find('1x') #checking for fp score
                    if l2>-1: lr=valfile.readline() #if found, reading next line
                    l3=lr.find('10x') #checking for fn score (if not found, -1 is returned)
                    if l3>-1: lr=valfile.readline() #if found, reading next line
                    l4=lr.find('11x') #checking for tp score
                    valfile.close() #closing file again
                else:
                    l1=1 #if number of lines is 4, setting all controls to positive (any value larger than -1 would be fine)
                    l2=1
                    l3=1
                    l4=1

                valfile=open(temppath+'/validation.txt', 'r') #once more opening temporary file with validation scores
                if l1==-1: tn=float(0) #if no tn score was identified before, setting to 0
                else:
                    tn=valfile.readline() #else, reading line
                    tn=float(tn.replace('0x', '').replace('\n', '')) #extracting tn score
                if l2==-1: fp=float(0) #if no fp score was identified before, setting to 0
                else:
                    fp=valfile.readline() #else, reading line
                    fp=float(fp.replace('1x', '').replace('\n', '')) #extracting fp score
                if l3==-1: fn=float(0) #if no fn score was identified before, setting to 0
                else:
                    fn=valfile.readline() #else, reading line
                    fn=float(fn.replace('10x', '').replace('\n', '')) #extracting fn score
                if l4==-1: tp=float(0) #if no tp score was identified before, setting to 0
                else:
                    tp=valfile.readline() #else, reading line
                    tp=float(tp.replace('11x', '').replace('\n', '')) #extracting tp score
                valfile.close() #closing file for the last time

                tn=5*(tp+fn)-fp #normalizing tn score
                foc=(tp+fp)/(tp+fn) #conservativeness factor

                if tp+fn>0 and tn+fp>0:

                    total=tp+tn+fp+fn #sum of scores

                    tpr=100*tp/total #tp rate in percent
                    tnr=100*tn/total #tn rate in percent
                    fpr=100*fp/total #fp ratte in percent
                    fnr=100*fn/total #fn rate in percent
                    if (tp+fn)>0: tlr=100*tp/(tp+fn) #rate of true positive predictions out of all positive observations
                    else: tlr=-9999
                    if (tn+fp)>0: tnlr=100*tn/(tn+fp) #rate of true negative predictions out of all negative observations
                    else: tnlr=-9999
                    mpr=tpr+fpr #total rate of positive predictions
                    mnr=tnr+fnr #total rate of negative predictions
                    tr=tnr+tpr #total rate of true predictions
                    fr=fnr+fpr #total rate of false predictions
                    opr=tpr+fnr #rate of positive observations
                    onr=tnr+fpr #rate of negative observations

                    if not tpr+fpr+fnr==0:
                        csi=tpr/(tpr+fpr+fnr) #critical success index
                    else: csi=-9999
                    if not (tpr+fnr)*(fnr+tnr)+(tpr+fpr)*(fpr+tnr)==0:
                        hss=(2*(tpr*tnr)-(fpr*fnr))/((tpr+fnr)*(fnr+tnr)+(tpr+fpr)*(fpr+tnr)) #Heidke skill score
                    else: hss=-9999

                    tprr=tpr/opr #rate of true positive rate
                    fprr=fpr/onr #rate of false positive rate
                    aucroc=(1-fprr)*tprr+fprr*tprr/2+(1-fprr)*(1-tprr)/2 #area under the roc curve
                    d2pc=pow(pow(1-tprr,2)+pow(fprr,2),0.5) #distance to perfect classification

                else:
                    tpr=-9999
                    tnr=-9999
                    fpr=-9999
                    fnr=-9999
                    tlr=-9999
                    tnlr=-9999
                    mpr=-9999
                    mnr=-9999
                    tr=-9999
                    fr=-9999
                    opr=-9999
                    onr=-9999
                    csi=-9999
                    hss=-9999
                    aucroc=-9999
                    d2pc=-9999

                if mstringi=='_hflow_dep': print>> fval, 'Validation scores for observed deposit'
                elif mstringi=='_hflow_max': print>> fval, 'Validation scores for observed impact area'

                print>> fval, 'Rate of positive observations\trOP\t%.1f%%' %opr
                print>> fval, 'Rate of negative observations\trON\t%.1f%%' %onr
                print>> fval, 'True positive rate\trTP\t%.1f%%' %tpr #writing rates to file
                print>> fval, 'True negative rate\trTN\t%.1f%%' %tnr
                print>> fval, 'False positive rate\trFP\t%.1f%%' %fpr
                print>> fval, 'False negative rate\trFN\t%.1f%%' %fnr
                print>> fval, 'True positive rate out of all positive observations\trTP/rOP\t%.1f%%' %tlr
                print>> fval, 'True negative rate out of all negative observations\trTN/rON\t%.1f%%' %tnlr
                print>> fval, 'Rate of total positive predictions\trMP\t%.1f%%' %mpr
                print>> fval, 'Rate of total negative predictions\trMN\t%.1f%%' %mnr
                print>> fval, 'Rate of true predictions\trT\t%.1f%%' %tr
                print>> fval, 'Rate of false predictions\trF\t%.1f%%' %fr
                print>> fval, 'Critical success index\tCSI\t%.3f%%' %csi
                print>> fval, 'Heidke Skill Score\tHSS\t%.3f%%' %hss
                print>> fval, 'Area under the ROC curve\tAUROC\t%.3f%%' %aucroc
                print>> fval, 'Distance to perfect classification\tD2PC\t%.3f%%' %d2pc
                print>> fval, 'Factor of conservativeness\tFoC\t%.3f%%' %foc
                print>> fval, ''

                if not mstringi=='_hflow_max': mstringj=mstringj+1 #updating counter

            if profile or impdef=='1' or depdef=='1': fval.close() #closing file


        # Creating profile file

            if profile: #if profile is defined:

                jprof=0
                jheader=['elev']
                grass.run_command('r.profile', input=pf+'_elev', output=temppath+'/fprof'+str(jprof)+'.txt',
                    coordinates=profile, null='-9999', quiet=True, overwrite=True) #elevation

                if hdeposit:

                    jprof=jprof+1
                    jheader.append('hdeposit')
                    grass.run_command('r.profile', input=pf+'_hdeposit', output=temppath+'/fprof'+str(jprof)+'.txt',
                        coordinates=profile, null='-9999', quiet=True, overwrite=True) #height of observed deposit

                for step in range(0,ntimesteps+1): #loop over all time steps:

                    if step<10: fill='000'+str(step) #formatting time step string
                    elif step<100: fill='00'+str(step)
                    elif step<1000: fill='0'+str(step)
                    else: fill=str(step)

                    for nprof in ('hflow', 'vflow', 'tflow', 'pflow', 'basechange'): #loop over all parameters:

                        if model=='1':

                            jprof=jprof+1
                            jheader.append(nprof+fill)
                            grass.run_command('r.profile', input=pf+'_'+nprof+fill, output=temppath+'/fprof'+str(jprof)+'.txt',
                                coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of parameter

                        elif model=='2':

                            jprof=jprof+1
                            jheader.append(nprof+'s'+fill)
                            grass.run_command('r.profile', input=pf+'_'+nprof+'s'+fill, output=temppath+'/fprof'+str(jprof)+'.txt',
                                coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of solid parameter
                            jprof=jprof+1
                            jheader.append(nprof+'f'+fill)
                            grass.run_command('r.profile', input=pf+'_'+nprof+'f'+fill, output=temppath+'/fprof'+str(jprof)+'.txt',
                                coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of fluid parameter

                fprof2=open(pf+'_results/'+pf+'_files/'+pf+'_profile.txt', 'w') #opening combined profile file for writing
                strprof=[] #string of file content
                for j in range(0,jprof+1): #loop over all parameters:
                    fprof=open(temppath+'/fprof'+str(j)+'.txt', 'r') #opening profile file for reading
                    strprof.append(fprof.readlines()) #writing entire file content to string
                    fprof.close() #closing file

                    if j==0: fprof2.write(jheader[j])
                    else: fprof2.write('\t'+jheader[j])
                fprof2.write('\n')

                for row in range(len(strprof[0])): #loop over all time steps:
                    strprof2=strprof[0][row].strip().split(' ')[0] #coordinate values
                    for j in range(0,jprof+1): #loop over all parameters:
                        strprof2=strprof2+'\t'+strprof[j][row].strip().split(' ')[1] #parameter values
                    fprof2.write(strprof2+'\n')

                fprof2.close() #closing file


        # Creating control point information file

            if ctrlpoints: #if control points are defined:

                fctrlpts=open(pf+'_results/'+pf+'_files/'+pf+'_ctrlpoints.txt', 'w') #opening control points information file for writing
                fctrlpts.write('id\tx\ty\tt\thflow\n') #writing header to file

                for ictrl in range(0,lctrlpts): #loop over all control points:

                    for step in range(0,ntimesteps+1): #loop over all time steps:

                        if step<10: fill='000'+str(step) #formatting time step string
                        elif step<100: fill='00'+str(step)
                        elif step<1000: fill='0'+str(step)
                        else: fill=str(step)

                        ctrlstring=grass.read_command('r.what', map=pf+'_hflow'+fill, separator='\t',
                            coordinates=xctrl[ictrl]+','+yctrl[ictrl], overwrite=True, quiet=True)
                            #reading maximum flow height at control point
                        ctrlstring=ctrlstring.replace('\t\t','\t').replace('\n','')

                        fctrlpts.write('C'+str(ictrl+1)+'\t'+str(step*float(tint))+'\t'+ctrlstring+'\n')

                    ctrlstring=grass.read_command('r.what', map=pf+'_hflow_max', separator='\t', coordinates=xctrl[ictrl]+','+yctrl[ictrl],
                        overwrite=True, quiet=True) #reading maximum flow height at control point
                    ctrlstring=ctrlstring.replace('\t\t','\t').replace('\n','')

                    fctrlpts.write('C'+str(ictrl+1)+'\tmax\t'+ctrlstring+'\n')
                        #writing point number and coordinates to file

                fctrlpts.close() #closing file

            stop = time.time() #storing time (end of main computation)
            comptime = stop - start #storing computational time in seconds

            timefile=open(filepath+pf+'_time.txt', 'w')
            timefile.write(str(comptime)) #writing computational time to file
            timefile.close()


    # Computing impact indicator indexs and writing input files for AIMEC

        else: #for multiple model runs:

            if int(sampling)>0: nruns=int(sampling) #number of model runs
            elif int(sampling)<0: nruns=int(sampling)*-1
            ncores=int(cores) #number of processors to be used

            os.mkdir(pf+'_results/'+pf+'_aimec') #creating directory for AIMEC input
            os.mkdir(pf+'_results/'+pf+'_aimec/depth') #creating directory for maximum flow height maps for AIMEC input
            os.mkdir(pf+'_results/'+pf+'_aimec/pressure') #creating directory for maximum kinetic energy maps for AIMEC input

            fparamt=open(temppath+'/fparamt.txt', 'w') #opening temporary input parameter file for writing
            faimecp=open(pf+'_results/'+pf+'_aimec/'+pf+'_aimec_params.txt', 'w') #opening parameter input file for AIMEC for writing

            if model=='1':
                print>> fparamt, 'nrun\tvhr\tvhem\trho\tphi\tmy\teps\trp\trd\tr0', #writing header to temporary input parameter file
                print>> faimecp, 'nrun, vhr, vhe, rho, phi, my, eps, rp, rd, r0', #writing header to parameter input file for AIMEC
                if baseparam:
                    print>> fparamt, '\tce'
                    print>> faimecp, ', ce'
                else:
                    print>> fparamt, ''
                    print>> faimecp, ''

            elif model=='2':
                print>> fparamt, 'nrun\tvhr\trhrs\tvhe\trhes\trhos\trhof\tvm\tphi\tdelta\tchi\txi\tvterm\tp\trep\tje\tnr\tnra\tad',
                print>> faimecp, 'nrun, vhr, rhrs, vhe, rhes, rhos, rhof, vm, phi, delta, chi, xi, vterm, p, rep, je, nr, nra, ad',
                if baseparam:
                    print>> fparamt, '\tce'
                    print>> faimecp, ', ce'
                else:
                    print>> fparamt, ''
                    print>> faimecp, ''

            if impdef=='1':
                grass.mapcalc('"_cimpactarea"=if("%s_impactarea">0,1,0)' %pf,
                    overwrite=True, quiet=True) #binary map of observed impact area
                grass.run_command('r.out.gdal', input='_cimpactarea', output=pf+'_results/'+pf+'_aimec/'+pf+'_impactarea.asc',
                    format='AAIGrid', overwrite=True) #exporting to ascii
                corrasc(pf+'_results/'+pf+'_aimec/'+pf+'_impactarea')

            if depdef=='1':
                grass.mapcalc('"_cdepositarea"=if("%s_hdeposit">0,1,0)' %pf,
                    overwrite=True, quiet=True) #binary map of observed height of deposit
                grass.run_command('r.out.gdal', input='_cdepositarea', output=pf+'_results/'+pf+'_aimec/'+pf+'_depositarea.asc',
                    format='AAIGrid', overwrite=True) #exporting to ascii
                corrasc(pf+'_results/'+pf+'_aimec/'+pf+'_depositarea')

            if profile: #creating profile file in directory for AIMEC input:

                fpaimec=open(pf+'_results/'+pf+'_aimec/'+pf+'_profile.xyz', 'w') #opening text file for writing
                profcoord=map(str, profile.split(',')) #splitting profile coordinate string
                proflength=len(profcoord) #reading number of profile coordinates
                for profnum in xrange(0,proflength,2): #loop over all coordinates:
                    profilex=profcoord[profnum] #extracting x coordinate
                    profiley=profcoord[profnum+1] #extracting y coordinate
                    profstring=grass.read_command('r.what', map=pf+'_elev', separator='\t', coordinates=profilex+','+profiley,
                        overwrite=True, quiet=True) #reading elevation value associated to profile coordinates
                    profstring=profstring.replace('\t\t','\t').replace('\n','')
                    print>> fpaimec, '%s' %profstring #writing profile string to text file
                fpaimec.close() #closing file


        # Preparing randomization of input raster maps

            if model=='2' and hrelease:
                rhrls=map(str, rhreleases.split(',')) #splitting string with ratio(s) of solid release height
                rhrlsmin=rhrls[0] #minimum
                if len(rhrls)==3: #maximum and third parameter
                    rhrlsmax=rhrls[1]
                    rhrls3=rhrls[2]
                elif len(rhrls)==2: rhrlsmax=rhrls[1]
                elif len(rhrls)==1: rhrlsmax=rhrls[0]
                else: ErrorMessage() #else, exiting with error message

            if hrelease and vhrelease:
                vhrl=map(str, vhrelease.split(',')) #splitting string with variation of release height
                vhrlmin=vhrl[0] #minimum
                if len(vhrl)==3: #maximum and third parameter
                    vhrlmax=vhrl[1]
                    vhrl3=vhrl[2]
                elif len(vhrl)==2: vhrlmax=vhrl[1]
                elif len(vhrl)==1: vhrlmax=vhrl[0]
                else: ErrorMessage() #else, exiting with error message
            else:
                vhrlmin='1'
                vhrlmax='1'
                vhrl3='1'

            if model=='2' and hentrmax:
                rhems=map(str, rhentrmaxs.split(',')) #splitting string with ratio(s) of maximum height of entrainment
                rhemsmin=rhems[0] #minimum
                if len(rhems)==3: #maximum and third parameter
                    rhemsmax=rhems[1]
                    rhems3=rhems[2]
                elif len(rhems)==2: rhemsmax=rhems[1]
                elif len(rhems)==1: rhemsmax=rhems[0]
                else: ErrorMessage() #else, exiting with error message

            if hentrmax and vhentrmax:
                vhem=map(str, vhentrmax.split(',')) #splitting string with variation of maximum height of entrainment
                vhemmin=vhem[0] #minimum
                if len(vhem)==2: #maximum and third parameter
                    vhemmax=vhem[1]
                    vhem3=vhem[2]
                elif len(vhem)==2: vhemmax=vhem[1]
                elif len(vhem)==1: vhemmax=vhem[0]
                else: ErrorMessage() #else, exiting with error message
            else:
                vhemmin='1'
                vhemmax='1'
                vhem3='1'

            if sampling=='0': #if controlled sampling is applied:

                njids=2
                nruns0=[]
                nruns=1

                if float(vhrlmin)==float(vhrlmax): nruns0.append(1)
                else: nruns0.append(int(vhrl3))
                nruns*=nruns0[0]

                if float(vhemmin)==float(vhemmax): nruns0.append(1)
                else: nruns0.append(int(vhem3))
                nruns*=nruns0[1]

                if model=='2' and hrelease and rhreleases:

                    njids+=1
                    if float(rhrlsmin)==float(rhrlsmax): nruns0.append(1)
                    else: nruns0.append(int(rhrls3))
                    nruns*=nruns0[2]

                if model=='2' and hentrmax and rhentrmaxs:

                    njids+=1
                    if float(rhemsmin)==float(rhemsmax): nruns0.append(1)
                    else: nruns0.append(int(rhems3))
                    nruns*=nruns0[njids-1]          

                for l in range(0,lmax):
                    if float(flowparam[3*l])==float(flowparam[3*l+1]): nruns0.append(1)
                    else: nruns0.append(int(flowparam[3*l+2]))
                    nruns*=nruns0[l+njids]

                for l in range(0,bmax):
                    if float(baseparam[3*l])==float(baseparam[3*l+1]): nruns0.append(1)
                    else: nruns0.append(int(baseparam[3*l+2]))
                    nruns*=nruns0[l+njids+lmax]

                jids=[0]
                for l in range(1,lmax+bmax+njids):jids.append(0)

            elif int(sampling)<0: #if OAT sampling is applied:

                nruns00=nruns
                nruns=0
                nvar=[]
                njids=2

                if not float(vhrlmin)==float(vhrlmax):
                    nvar.append(1)
                    nruns+=nruns00
                else: nvar.append(0)

                if not float(vhemmin)==float(vhemmax):
                    nvar.append(1)
                    nruns+=nruns00
                else: nvar.append(0)

                if model=='2' and hrelease and rhreleases:

                    njids+=1
                    if not float(rhrlsmin)==float(rhrlsmax):
                        nvar.append(1)
                        nruns+=nruns00
                    else: nvar.append(0)

                if model=='2' and hentrmax and rhentrmaxs:

                    njids+=1
                    if not float(rhemsmin)==float(rhemsmax):
                        nvar.append(1)
                        nruns+=nruns00
                    else: nvar.append(0)

                for l in range(0,lmax):
                    if not float(flowparam[3*l])==float(flowparam[3*l+1]):
                        nvar.append(1)
                        nruns+=nruns00
                    else: nvar.append(0)

                for l in range(0,bmax):
                    if not float(baseparam[3*l])==float(baseparam[3*l+1]):
                        nvar.append(1)
                        nruns+=nruns00
                    else: nvar.append(0)

                lnrun=0
                ltest=0

            for jid in range(1, nruns+1): #loop over predefined number of randomized parameter combinations:

                if jid<10: jfill='00000'+str(jid) #formatting model run string
                elif jid<100: jfill='0000'+str(jid)
                elif jid<1000: jfill='000'+str(jid)
                elif jid<10000: jfill='00'+str(jid)
                elif jid<100000: jfill='0'+str(jid)
                else: jfill=str(jid)

                if sampling=='0': #if controlled sampling is applied:

                    jidctrl=0
                    for l in range(0,lmax+bmax+njids):
                        if l==0 or jids[l]==0 or jidctrl==1: jids[l]+=1
                        if jids[l]>nruns0[l]:
                           jids[l]=1
                           jidctrl=1
                        else: jidctrl=0


            # Randomizing input raster maps

                if sampling=='0': #for controlled sampling:

                    if float(vhrlmin)==float(vhrlmax): vhrl=float(vhrlmin)
                    else: vhrl=float(vhrlmin)+float(jids[0]-1)/float(nruns0[0]-1)*(float(vhrlmax)-float(vhrlmin))

                elif int(sampling)>0: #for random sampling:

                    vhrl=round(random.uniform(float(vhrlmin), float(vhrlmax)),2) #variation of release height

                else: #if OAT sampling is applied:

                    if ltest<lmax+bmax+njids:
                        while nvar[ltest]==0 and ltest<lmax+bmax+njids:
                            ltest+=1
                            lnrun=0

                    if float(vhrlmin)==float(vhrlmax) or not ltest==0: vhrl=float(vhrl3)
                    else: vhrl=float(vhrlmin)+float(lnrun)/float(nruns00-1)*(float(vhrlmax)-float(vhrlmin))

                vhrl=str(vhrl)

                if sampling=='0': #for controlled sampling:

                    if float(vhemmin)==float(vhemmax): vhem=float(vhemmin)
                    else: vhem=float(vhemmin)+float(jids[1]-1)/float(nruns0[1]-1)*(float(vhemmax)-float(vhemmin))

                elif int(sampling)>0: #for random sampling:

                    vhem=round(random.uniform(float(vhemmin), float(vhemmax)),2) #variation of maximum height of entrainment

                else: #if OAT sampling is applied:

                    if float(vhemmin)==float(vhemmax) or not ltest==1: vhem=float(vhem3)
                    else: vhem=float(vhemmin)+float(lnrun)/float(nruns00-1)*(float(vhemmax)-float(vhemmin))

                vhem=str(vhem)

                nhrelease=pf+'_hflow0000'+str(jid)
                if model=='1': #variables for names of input raster maps for model run:
                    nhentrmax=pf+'_hentrmax'+str(jid)
                elif model=='2':
                    nhreleases=pf+'_hflows0000'+str(jid)
                    nhreleasef=pf+'_hflowf0000'+str(jid)
                    nhentrmaxs=pf+'_hentrmaxs'+str(jid)
                    nhentrmaxf=pf+'_hentrmaxf'+str(jid)

                if hrelease: #if release height is given:
                    grass.mapcalc('"%s"="%s_hflow0000"*%s' %(nhrelease,pf,vhrl), overwrite=True, quiet=True) #release height map
                else: grass.mapcalc('"%s"=0' %nhrelease, overwrite=True, quiet=True)

                if model=='1' and hentrmax: #if maximum height of entrainment is given for Voellmy-Salm one-phase model:
                    grass.mapcalc('"%s"="%s_hentrmax"*%s' %(nhentrmax,pf,vhem), overwrite=True, quiet=True)
                        #maximum height of entrainment map
                elif model=='1': grass.mapcalc('"%s"=0' %nhentrmax, overwrite=True, quiet=True)

                if model=='2' and hrelease and rhreleases: #if release heights are defined by total height and ratio of solid:

                    if sampling=='0': #for controlled sampling:

                        if float(rhrlsmin)==float(rhrlsmax): rhrls=float(rhrlsmin)
                        else: rhrls=float(rhrlsmin)+float(jids[2]-1)/float(nruns0[2]-1)*(float(rhrlsmax)-float(rhrlsmin))

                    elif int(sampling)>0: #for random sampling:

                        rhrls=random.uniform(float(rhrlsmin), float(rhrlsmax)) #randomizing ratio of solid release height

                    else: #if OAT sampling is applied:

                        if float(rhrlsmin)==float(rhrlsmax) or not ltest==2: rhrls=float(rhrls3)
                        else: rhrls=float(rhrlsmin)+float(lnrun)/float(nruns00-1)*(float(rhrlsmax)-float(rhrlsmin))

                    rhrls=str(rhrls) #constraining ratio and converting to string

                    grass.mapcalc('"%s"="%s_hflow0000"*%s*%s' %(nhreleases,pf,rhrls,vhrl),
                        overwrite=True, quiet=True) #solid release height map
                    grass.mapcalc('"%s"="%s_hflow0000"*(1-%s)*%s' %(nhreleasef,pf,rhrls,vhrl),
                        overwrite=True, quiet=True) #fluid release height map

                elif model=='2':
                    grass.mapcalc('"%s"="%s_hflows0000"' %(nhreleases, pf), overwrite=True, quiet=True)
                    grass.mapcalc('"%s"="%s_hflowf0000"' %(nhreleasef, pf), overwrite=True, quiet=True)
                    rhrls='-9999'

                if model=='2' and hentrmax and rhentrmaxs:
                    #if maximum heights of entrainment are defined by total height and ratio of solid:

                    if sampling=='0': #for controlled sampling:

                        if float(rhemsmin)==float(rhemsmax): rhems=float(rhemsmin)
                        else: rhems=float(rhemsmin)+float(jids[njids-1]-1)/float(nruns0[njids-1]-1)*(float(rhemsmax)-float(rhemsmin))

                    elif int(sampling)>0: #for random sampling:

                        rhems=random.uniform(float(rhemsmin), float(rhemsmax)) #randomizing ratio of maximum solid height of entrainment

                    else: #if OAT sampling is applied:

                        if float(rhemsmin)==float(rhemsmax) or not ltest==njids-1: rhems=float(rhems3)
                        else: rhems=float(rhemsmin)+float(lnrun)/float(nruns00-1)*(float(rhemsmax)-float(rhemsmin))

                    rhems=str(rhems) #constraining ratio and converting to string

                    grass.mapcalc('"%s"="%s_hentrmax"*%s*%s' %(nhentrmaxs,pf,rhems,vhem),
                        overwrite=True, quiet=True) #maximum height of solid entrainment map
                    grass.mapcalc('"%s"="%s_hentrmax"*(1-%s)*%s' %(nhentrmaxf,pf,rhems,vhem),
                        overwrite=True, quiet=True) #maximum height of fluid entrainment map

                elif model=='2':
                    if entrainment==1: grass.mapcalc('"%s"="%s_hentrmaxs"' %(nhentrmaxs, pf), overwrite=True, quiet=True)
                    if entrainment==1: grass.mapcalc('"%s"="%s_hentrmaxf"' %(nhentrmaxf, pf), overwrite=True, quiet=True)
                    rhems='-9999'

                if model=='1': 

                    if releasemass==1: grass.run_command('r.out.gdal', input=nhrelease, output=ascpath+nhrelease+'.asc', format='AAIGrid',
                        overwrite=True) #exporting release height map to ascii
                    if entrainment==1: grass.run_command('r.out.gdal', input=nhentrmax, output=ascpath+nhentrmax+'.asc', format='AAIGrid',
                        overwrite=True) #exporting maximum height of entrainment map to ascii

                    if releasemass==1: corrasc(ascpath+nhrelease)
                    if entrainment==1: corrasc(ascpath+nhentrmax)

                elif model=='2':

                    if releasemass==1: grass.run_command('r.out.gdal', input=nhreleases, output=ascpath+nhreleases+'.asc', format='AAIGrid',
                        overwrite=True) #exporting solid release height map to ascii
                    if releasemass==1: grass.run_command('r.out.gdal', input=nhreleasef, output=ascpath+nhreleasef+'.asc', format='AAIGrid',
                        overwrite=True) #exporting fluid release height map to ascii
                    if entrainment==1: grass.run_command('r.out.gdal', input=nhentrmaxs, output=ascpath+nhentrmaxs+'.asc', format='AAIGrid',
                        overwrite=True) #exporting maximum height of solid entrainment map to ascii
                    if entrainment==1: grass.run_command('r.out.gdal', input=nhentrmaxf, output=ascpath+nhentrmaxf+'.asc', format='AAIGrid',
                        overwrite=True) #exporting maximum height of fluid entrainment map to ascii

                    if releasemass==1: corrasc(ascpath+nhreleases)
                    if releasemass==1: corrasc(ascpath+nhreleasef)
                    if entrainment==1: corrasc(ascpath+nhentrmaxs)
                    if entrainment==1: corrasc(ascpath+nhentrmaxf)


            # Writing flow parameters to file

                p1file=open(temppath+'/param'+str(jid)+'.txt', 'w') #opening parameter files
                gt=[]
                if baseparam: bp=[]

                if sampling=='0': #if controlled sampling is applied:

                    for l in range(0,lmax):
                        if float(flowparam[3*l])==float(flowparam[3*l+1]): gt.append(float(flowparam[3*l]))
                        else: gt.append(float(flowparam[3*l])+float(jids[l+njids]-1)
                            /float(nruns0[l+njids]-1)*(float(flowparam[3*l+1])-float(flowparam[3*l])))

                    for l in range(0,bmax):
                        if float(baseparam[3*l])==float(baseparam[3*l+1]): bp.append(float(baseparam[3*l]))
                        else: bp.append(float(baseparam[3*l])+float(jids[l+njids+lmax]-1)
                            /float(nruns0[l+njids+lmax]-1)*(float(baseparam[3*l+1])-float(baseparam[3*l])))

                elif int(sampling)>0: #if random sampling is applied:

                    for l in range(0,lmax): gt.append(random.uniform(float(flowparam[2*l]), float(flowparam[2*l+1])))
                    for l in range(0,bmax): bp.append(random.uniform(float(baseparam[2*l]), float(baseparam[2*l+1])))
                        #randomizing flow and basal surface parameters

                else: #if OAT sampling is applied:

                    for l in range(0,lmax):
                        if float(flowparam[3*l])==float(flowparam[3*l+1]) or not ltest==l+njids: gt.append(float(flowparam[3*l+2]))
                        else: gt.append(float(flowparam[3*l])+float(lnrun)
                             /float(nruns00-1)*(float(flowparam[3*l+1])-float(flowparam[3*l])))

                    for l in range(0,bmax):
                        if float(baseparam[3*l])==float(baseparam[3*l+1]) or not ltest==l+njids: bp.append(float(baseparam[3*l+2]))
                        else: bp.append(float(baseparam[3*l])+float(lnrun)
                             /float(nruns00-1)*(float(baseparam[3*l+1])-float(baseparam[3*l])))

                    lnrun+=1
                    if lnrun==nruns00:
                        lnrun=0
                        ltest+=1

                print>> p1file, mainmapset #name of main mapset
                print>> p1file, '1' #identifier for multiple model runs
                print>> p1file, model #model
                print>> p1file, topo #topography

                print>> p1file, funcctrl[0] #control for correction of flow height
                print>> p1file, funcctrl[1] #control for diffusion control
                print>> p1file, funcctrl[2] #control for conservation of volume
                print>> p1file, funcctrl[3] #control for surface control
                print>> p1file, funcctrl[4] #control for entrainment and deposition
                print>> p1file, funcctrl[5] #control for stopping

                if baseparam and (not funcctrl[4]=='0'):
                    if not bmax==1: ErrorMessage()
                    #making sure that basal surface parameters are provided correctly

                print>> p1file, str(releasemass) #control for (solid) release height
                if model=='2': print>> p1file, str(releasemass) #control for fluid release height
                print>> p1file, str(entrainment) #control for maximum height of (solid) entrainment
                if model=='2': print>> p1file, str(entrainment) #control for maximum height of fluid entrainment
                if funcctrl[4]=='1' and centr: print>> p1file, '1' #control for entrainment coefficient map
                else: print>> p1file, '0' 
                if delta: print>> p1file, '1' #control for basal friction angle map
                else: print>> p1file, '0'
                if hydrograph or hydrocoords: print>> p1file, '1' #control for hydrograph
                else: print>> p1file, '0'
                if topo=='2': print>> p1file, aoi #name of area of interest map
                print>> p1file, elevation #name of elevation map
                if model=='1' and releasemass==1: print>> p1file, nhrelease #name of release height map
                elif model=='2' and releasemass==1: print>> p1file, nhreleases #name of solid release height map
                if model=='2' and releasemass==1: print>> p1file, nhreleasef #name of fluid release height map
                if model=='1' and entrainment==1: print>> p1file, nhentrmax #name of maximum height of entrainment map
                elif model=='2' and entrainment==1: print>> p1file, nhentrmaxs #name of maximum height of solid entrainment map
                if model=='2' and entrainment==1: print>> p1file, nhentrmaxf #name of maximum height of fluid entrainment map
                if funcctrl[4]=='1' and centr: print>> p1file, centr #name of entrainment coefficient map
                if delta: print>> p1file, delta #name of basal friction angle map

                if hydrograph or hydrocoords:
                    print>> p1file, hyd_nin #number of input hydrographs
                    print>> p1file, hyd_nout #number of output hydrographs
                    for hydi in range(0,hyd_nin):
                        print>> p1file, hydrograph[hydi] #path to hydrograph file
                        print>> p1file, str(hydlines[hydi]) #number of hydrograph time steps 
                    for hydi in range(0,hyd_nin+hyd_nout):
                        print>> p1file, hydrocoords[4*hydi] #x coordinate of hydrograph
                        print>> p1file, hydrocoords[4*hydi+1] #y coordinate of hydrograph
                        print>> p1file, hydrocoords[4*hydi+2] #length of hydrograph profile
                        print>> p1file, hydrocoords[4*hydi+3] #aspect of hydrograph profile
                else:
                    hyd_nin=0
                    hyd_nout=0

                print>> p1file, lmax #number of flow parameters
                for l in range(0,lmax): print>> p1file, round(gt[l],10) #flow parameters
                if baseparam and funcctrl[4]!='0':
                    for l in range(0,bmax): print>> p1file, bp[l] #basal surface parameters
                elif not baseparam and funcctrl[4]!='0':
                    for l in range(0,1): print>> p1file, '-9999'

                print>> p1file, imparamc #threshold of flow height (for computation)
                print>> p1file, imparam[0] #threshold of flow height (for display)
                print>> p1file, imparam[1] #threshold of flow kinetic energy
                print>> p1file, imparam[2] #threshold of flow pressure

                print>> p1file, tint #time for writing output
                print>> p1file, tstop #process duration at which to stop
                print>> p1file, cfl[0] #cfl criterion
                print>> p1file, cfl[1] #maximum length of time step

                print>> p1file, '%s_' %pf #prefix
                print>> p1file, '%s_results/%s_ascii/' %(pf,pf) #path and prefix for storing output maps
                print>> p1file, '%s_results/%s_files/' %(pf,pf) #path and prefix for storing output files

                p1file.close() #closing parameter file

                fparamtstring=jfill
                if model=='1': fparamtstring=fparamtstring+'\t'+vhrl+'\t'+vhem
                elif model=='2': fparamtstring=fparamtstring+'\t'+vhrl+'\t'+rhrls+'\t'+vhem+'\t'+rhems
                for l in range(0,lmax): fparamtstring=fparamtstring+'\t'+str(gt[l])
                for l in range(0,bmax): fparamtstring=fparamtstring+'\t'+str(bp[l])
                print>> fparamt, fparamtstring #writing parameters to temporary input parameter file


            # Creating batch file

                os.mkdir(temppath+'/tmp%s' % jid) #creating directory for batch file
                strtmp=temppath+'/tmp%s/batch' % jid #file name for batch file
                out=file(strtmp+str(jid),'w') #creating batch file
                os.system('rm -rf '+locpath+'/map%s' %jid) #removing old mapset for model run
                grass.run_command('g.mapset', flags='c', mapset='map%s' %jid) #creating new mapset for model run
                grass.run_command('g.mapsets', mapset=ambvars.MAPSET, operation='add') #making original mapset active

                grass.run_command("g.mapset", mapset=ambvars.MAPSET) #switching back to original mapset
                os.system('mkdir '+locpath+'/map%s/.tmp/rtemp' %jid) #creating directory in mapset for model run
                os.environ['PATH']+=os.pathsep+os.path.join(temppath+'/tmp%s') %jid #adding path to batch file

                print >> out,"""#!/bin/bash
export jid=%s
export SHELL=\"/bin/bash\"
export cellsize=%s
export rnorth=%s
export rsouth=%s
export rwest=%s
export reast=%s
export rtemp=%s
export GRASS_BATCH_JOB=%s/r.avaflow.mult
cp %s/*.txt %s/map%s/.tmp/rtemp/
%s -text %s/map%s
unset GRASS_BATCH_JOB""" % (jid,cellsize,rnorth,rsouth,rwest,reast,temppath,scriptpath,temppath,locpath,jid,bingrass,locpath,jid)
    #creating batch file

                out.close() #closing batch file
                fd=os.open(strtmp+str(jid),os.O_RDONLY) #opening batch file
                os.fchmod(fd,0755) #making batch file executable
                os.close( fd ) #closing batch file

            fparamt.close() #closing input parameter file


        # Executing batch processing

            start_batch = time.time() #storing time (start of multi-core processing)

            neff=min(ncores,nruns)
            threadList=range(1,neff+1)
            nameList=range(1,nruns+1)
            threads=[]
            for tName in threadList:
                thread=myThread(tName, workQueue)
                thread.start()
                threads.append(thread)
            queueLock.acquire()
            for word in nameList:
                workQueue.put(word)
            queueLock.release()
            while not workQueue.empty():
               pass
            global exitFlag
            exitFlag=1
            for t in threads:
                t.join()
            print
            print 'Batch processing completed.'
            print

            for jid in range(1,nruns+1): os.system('rm -rf '+locpath+'/map'+str(jid)) #removing mapsets for all model runs

            fval=open(pf+'_results/'+pf+'_files/'+pf+'_validation.txt', 'w') #opening validation file for writing
            if model=='1': fvalstring='nrun\tHmax\tHmaxd\tVmax\tTmax\tPmax\tHefin\tVE'  #adding header to string for validation file:
            elif model=='2': fvalstring='nrun\tHmax\tHmaxd\tVsmax\tVfmax\tTmax\tPmax\tHefin\tVEs\tVEf'
            if profile:
                fvalstring=fvalstring+'\t'+'Ldm\tLim'
                if impdef=='1': fvalstring=fvalstring+'\t'+'dLi'
                if depdef=='1': fvalstring=fvalstring+'\t'+'dLd'

            if impdef=='1':
                fvalstring=fvalstring+'\t'+'rTPi\trTNi\trFPi\trFNi\tCSIi\tHSSi\tAUROCi\tD2PCi\tFoC'
                grass.mapcalc('"_obsbini"=if("%s_impactarea">0,1,if("%s_impactarea"==0,0,null()))' %(pf, pf),
                overwrite=True, quiet=True) #processed map of observed impact area

            if depdef=='1':
                fvalstring=fvalstring+'\t'+'rTPd\trTNd\trFPd\trFNd\tCSId\tHSSd\tAUROCd\tD2PCd\tFoC'
                grass.mapcalc('"_obsbind"=if("%s_hdeposit">=%s,1,if("%s_hdeposit"==0,0,null()))' %(pf, imparam[0], pf),
                overwrite=True, quiet=True) #processed map of height of observed deposit

            print>> fval, fvalstring #writing header to validation file


        # Impact and deposition indicator indices and validation file

            fparamt=open(temppath+'/fparamt.txt', 'r') #opening temporary input parameter file for reading
            fparami=open(pf+'_results/'+pf+'_files/'+pf+'_params.txt', 'w') #opening input parameter file for reading
            print>> fparami, fparamt.readline().replace('\n','') #writing header of temporary input parameter file to input parameter file

            for j in range(0,3): #loop over all output parameters for wich impact indicator index should be derived:
                grass.mapcalc('"%s_iii%s"=0' %(pf,mstring[j]), overwrite=True, quiet=True) #initializing impact indicator index map
                grass.mapcalc('"%s_dii"=0' %pf, overwrite=True, quiet=True) #initializing deposit indicator index map

            nsuccess=0 #intitalizing counter for number of successful simulations
            for jid in nameList: #loop over all model runs:

                ftimesteps=open(pf+'_results/'+pf+'_files/'+pf+'_nout'+str(jid)+'.txt', 'r')
                    #opening file with number of time steps and success
                ftimesteps.readline() #number of time steps is not needed
                csuccess=ftimesteps.readline() #reading control for success of simulation
                csuccess=int(csuccess.replace('\n', '')) #removing newline
                vol_entr=ftimesteps.readline() #reading entrained (solid) volume
                vol_entr=float(vol_entr.replace('\n', '')) #removing newline
                vol_entr2=ftimesteps.readline() #reading entrained fluid volume
                vol_entr2=float(vol_entr2.replace('\n', '')) #removing newline
                ftimesteps.close() #closing file with number of time steps

                if csuccess==1: fvalstring=str(jid) #writing number of model run to string

                if model=='1': mstringlist=['_hflow_max','_hflow_fin','_vflow_max','_tflow_max','_pflow_max','_basechange_fin']
                elif model=='2':
                    mstringlist=['_hflow_max','_hflow_fin','_vflows_max','_vflowf_max','_tflow_max','_pflow_max','_basechange_fin']

                for mstringi in mstringlist:

                    grass.run_command('r.in.gdal', input=ascpath+pf+mstringi+str(jid)+'.asc',
                        output=pf+mstringi+str(jid), overwrite=True) #importing map
                    if csuccess==1:
                        maxval=grass.raster_info(pf+mstringi+str(jid))['max'] #maximum value
                        fvalstring=fvalstring+'\t'+str(maxval) #adding maximum value to string
                        if mstringi=='_tflow_max' and maxval==0: csuccess=0 #setting control for success to negative, if necessary

                fvalstring=fvalstring+'\t'+str(vol_entr) #adding entrained (solid) volume to string
                if model=='2': fvalstring=fvalstring+'\t'+str(vol_entr2) #adding entrained fluid volume to string

                if basechange==1: grass.mapcalc('"%s_hflow_dep%s"=if("%s_basechange_fin%s">0,"%s_basechange_fin%s",0)'
                    %(pf, str(jid), pf, str(jid), pf, str(jid)), overwrite=True, quiet=True)
                else: grass.mapcalc('"%s_hflow_dep%s"="%s_hflow_fin%s"' %(pf, str(jid), pf, str(jid)), overwrite=True, quiet=True)
                    #simulated height of deposition map

                flinet=fparamt.readline() #reading line of temporary input parameter file

                if csuccess==1: #if simulation was successful:

                    nsuccess=nsuccess+1 #updating number of successful simulations

                    grass.mapcalc('"%s_iis%s"=if("%s_hflow_max%s">%s,1,0)' %(pf,jfill,pf,str(jid),imparam[0]),
                        overwrite=True, quiet=True) #impact indicator score map

                    grass.mapcalc('"%s_dis%s"=if("%s_hflow_dep%s">%s,1,0)' %(pf,jfill,pf,str(jid),imparam[0]),
                        overwrite=True, quiet=True) #deposit indicator score map

                    for j in range(0,3): #loop over all output parameters for wich impact indicator index should be derived:

                        grass.mapcalc('"%s_iii%s"=if("%s%s_max%s">%s,"%s_iii%s"+1,"%s_iii%s")'
                            %(pf,mstring[j],pf,mstring[j],str(jid),imparam[j],pf,mstring[j],pf,mstring[j]),
                            overwrite=True, quiet=True) #updating impact indicator index map

                    grass.mapcalc('"%s_dii"=if("%s_hflow_dep%s">%s,"%s_dii"+1,"%s_dii")' %(pf,pf,str(jid),imparam[0], pf, pf),
                        overwrite=True, quiet=True) #updating deposit indicator index map

                    if profile: #if profile is defined:

                        rstring='_hflow0000'
                        grass.run_command('r.profile', input=pf+rstring+str(jid),
                            output=temppath+'/fhrelease.txt', coordinates=profile, null='-9999', quiet=True, overwrite=True)
                            #profile of release height

                        fprof=open(temppath+'/fhrelease.txt', 'r') #opening profile of release height for reading
                        flines = sum(1 for line in fprof) #counting number of lines
                        fprof.close() #closing file

                        flowline_start=0.0 #setting initial value for start to start point of profile
                        if releasemass==1: #if release height is defined:

                            fprof=open(temppath+'/fhrelease.txt') #again, opening profile file for reading
                            for j in range(1,flines): #loop over all lines:
                                fline=fprof.readline() #reading and splitting line
                                fline=map(str, fline.split(' '))
                                if float(fline[2].replace('\n',''))>0:
                                    flowline_start=float(fline[1]) #identifying profile position at start of flow
                                    break
                            fprof.close() #closing file

                        if impdef=='1': #identifying profile position at terminus of observed flow:

                            mstringj=0 #initializing counter
                            grass.run_command('r.profile', input=pf+'_impactarea', output=temppath+'/fimp.txt',
                                coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of map

                            flowline_stopimp = None
                            fprof=open(temppath+'/fimp.txt') #opening profile for reading
                            for j in range(1,flines): #loop over all lines:
                                fline=fprof.readline() #reading and splitting line
                                fline=map(str, fline.split(' '))
                                if float(fline[2].replace('\n',''))==1 or (not flowline_stopimp and j==flines-1):
                                    flowline_stopimp=float(fline[1]) #identifying profile position at terminus of observed flow
                            fprof.close() #closing file

                        if depdef=='1': #identifying profile position at terminus of observed flow:

                            mstringj=0 #initializing counter
                            grass.run_command('r.profile', input=pf+'_hdeposit', output=temppath+'/fobs.txt',
                                coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of map

                            flowline_stopdep = None
                            fprof=open(temppath+'/fobs.txt') #opening profile for reading
                            for j in range(1,flines): #loop over all lines:
                                fline=fprof.readline() #reading and splitting line
                                fline=map(str, fline.split(' '))
                                if float(fline[2].replace('\n',''))>=float(imparam[0]) or (not flowline_stopdep and j==flines-1):
                                    flowline_stopdep=float(fline[1]) #identifying profile position at terminus of observed flow
                            fprof.close() #closing file

                        mstringj=0 #initializing counter
                        for mstringi in ['_hflow_max','_hflow_dep']:

                            grass.run_command('r.profile', input=pf+mstringi+str(jid), output=temppath+'/f'+mstringi+'.txt',
                                coordinates=profile, null='-9999', quiet=True, overwrite=True) #profile of map

                            flowline_stop = None
                            fprof=open(temppath+'/f'+mstringi+'.txt') #opening profile for reading
                            for j in range(1,flines): #loop over all lines:
                                fline=fprof.readline() #reading and splitting line
                                fline=map(str, fline.split(' '))
                                if float(fline[2].replace('\n',''))>=float(imparam[mstringj]) or (not flowline_stop and j==flines-1):
                                    flowline_stop=float(fline[1]) #identifying profile position at termination of flow
                            fprof.close() #closing file
                            maxlength=flowline_stop-flowline_start #flow length
                            fvalstring=fvalstring+'\t'+str(maxlength) #adding flow length to string
                            if impdef=='1' and mstringi=='_hflow_max': flowline_diffimp=flowline_stop-flowline_stopimp
                            elif depdef=='1' and mstringi=='_hflow_dep': flowline_diffdep=flowline_stop-flowline_stopdep
                            if not mstringi=='_hflow_max': mstringj=mstringj+1 #updating counter

                        if impdef=='1': fvalstring=fvalstring+'\t'+str(flowline_diffimp) #adding difference in flow length to string
                        if depdef=='1': fvalstring=fvalstring+'\t'+str(flowline_diffdep) #adding difference in flow length to string

                    if impdef=='1' and depdef=='1': mstringlist2=['_hflow_max','_hflow_dep']
                        #raster maps to be processed for validation scores
                    elif impdef=='1': mstringlist2=['_hflow_max']
                    elif depdef=='1': mstringlist2=['_hflow_dep']
                    else: mstringlist2=[]

                    mstringj=0
                    for mstringi in mstringlist2: #loop over all maps to be processed:

                        grass.mapcalc('"_simbin"=if("%s%s%s">=%s,1,0)' %(pf, mstringi, str(jid), imparam[mstringj]),
                            overwrite=True, quiet=True) #processed map of simulated impact area or deposit

                        if mstringi=='_hflow_max': grass.mapcalc('"_eval"="_simbin"+10*"_obsbini"', overwrite=True, quiet=True)
                            #combination of observed and simulated impact areas
                        elif mstringi=='_hflow_dep': grass.mapcalc('"_eval"="_simbin"+10*"_obsbind"', overwrite=True, quiet=True)
                            #combination of observed and simulated deposits

                        grass.run_command('r.stats', flags='c'+'n', separator='x', input='_eval', output='%s/validation.txt'
                            %temppath, quiet=True, overwrite=True) #scores tp=11, tn=0, fp=1, fn=10

                        valfile=open(temppath+'/validation.txt', 'r') #opening raw file with validation scores
                        valfile_lines=sum(1 for line in valfile) #reading number of lines
                        valfile.close() #closing file

                        if valfile_lines<4: #if number of lines is smaller than 4 (at least one of the scores is 0):
                            valfile=open(temppath+'/validation.txt', 'r') #opening file again
                            lr=valfile.readline() #reading first line
                            l1=lr.find('0x') #checking for tn score
                            if l1>-1: lr=valfile.readline() #if found, reading next line
                            l2=lr.find('1x') #checking for fp score
                            if l2>-1: lr=valfile.readline() #if found, reading next line
                            l3=lr.find('10x') #checking for fn score (if not found, -1 is returned)
                            if l3>-1: lr=valfile.readline() #if found, reading next line
                            l4=lr.find('11x') #checking for tp score
                            valfile.close() #closing file again
                        else:
                            l1=1 #if number of lines is 4, setting all controls to positive (any value larger than -1 would be fine)
                            l2=1
                            l3=1
                            l4=1

                        valfile=open(temppath+'/validation.txt', 'r') #once more opening temporary file with validation scores
                        if l1==-1: tn=float(0) #if no tn score was identified before, setting to 0
                        else:
                            tn=valfile.readline() #else, reading line
                            tn=float(tn.replace('0x', '').replace('\n', '')) #extracting tn score
                        if l2==-1: fp=float(0) #if no fp score was identified before, setting to 0
                        else:
                            fp=valfile.readline() #else, reading line
                            fp=float(fp.replace('1x', '').replace('\n', '')) #extracting fp score
                        if l3==-1: fn=float(0) #if no fn score was identified before, setting to 0
                        else:
                            fn=valfile.readline() #else, reading line
                            fn=float(fn.replace('10x', '').replace('\n', '')) #extracting fn score
                        if l4==-1: tp=float(0) #if no tp score was identified before, setting to 0
                        else:
                            tp=valfile.readline() #else, reading line
                            tp=float(tp.replace('11x', '').replace('\n', '')) #extracting tp score
                        valfile.close() #closing file for the last time

                        tn=5*(tp+fn)-fp #normalizing tn score
                        foc=(tp+fp)/(tp+fn) #conservativeness factor

                        if tn+fp>0 and tp+fn>0:

                            total=tp+tn+fp+fn #sum of scores

                            tpr=100*tp/total #tp rate in percent
                            tnr=100*tn/total #tn rate in percent
                            fpr=100*fp/total #fp rate in percent
                            fnr=100*fn/total #fn rate in percent
                            opr=tpr+fnr #rate of positive observations
                            onr=tnr+fpr #rate of negative observations

                            if not tpr+fpr+fnr==0:
                                csi=tpr/(tpr+fpr+fnr) #critical success index
                            else: csi=-9999
                            if not (tpr+fnr)*(fnr+tnr)+(tpr+fpr)*(fpr+tnr)==0:
                                hss=(2*(tpr*tnr)-(fpr*fnr))/((tpr+fnr)*(fnr+tnr)+(tpr+fpr)*(fpr+tnr)) #Heidke skill score
                            else: hss=-9999

                            tprr=tpr/opr #rate of true positive rate
                            fprr=fpr/onr #rate of false positive rate
                            aucroc=(1-fprr)*tprr+fprr*tprr/2+(1-fprr)*(1-tprr)/2 #area under the roc curve
                            d2pc=pow(pow(1-tprr,2)+pow(fprr,2),0.5) #distance to perfect classification

                        else:
                            tpr=-9999
                            tnr=-9999
                            fpr=-9999
                            fnr=-9999
                            csi=-9999
                            hss=-9999
                            aucroc=-9999
                            d2pc=-9999

                        fvalstring=fvalstring+'\t'+str(tpr)+'\t'+str(tnr)+'\t'+str(fpr)+'\t'+str(fnr)
                        fvalstring=fvalstring+'\t'+str(csi)+'\t'+str(hss)+'\t'+str(aucroc)+'\t'+str(d2pc)+'\t'+str(foc)
                            #adding evaluation results to string

                        if not mstringi=='_hflow_max': mstringj=mstringj+1 #updating counter

                    print>> fval, fvalstring #writing string for model run to text file


                # Input for AIMEC

                    if jid<10: jfill='00000'+str(jid) #formatting model run string
                    elif jid<100: jfill='0000'+str(jid)
                    elif jid<1000: jfill='000'+str(jid)
                    elif jid<10000: jfill='00'+str(jid)
                    elif jid<100000: jfill='0'+str(jid)
                    else: jfill=str(jid)

                    flinep=flinet.replace('\t',';').replace('\n','') #preparing line for writing to parameter input file  for AIMEC
                    print>> faimecp, flinep #writing line of input parameter file to parameter input file for AIMEC

                    grass.mapcalc('"_hflowii"=if("%s_hflow_max%s">=%s,"%s_hflow_max%s",0)' %(pf,str(jid),imparam[0],pf,str(jid)),
                        overwrite=True, quiet=True) #constraining maximum flow height map
                    grass.mapcalc('"_tflowii"=if("%s_tflow_max%s">=%s,"%s_tflow_max%s",0)' %(pf,str(jid),imparam[1],pf,str(jid)),
                        overwrite=True, quiet=True) #constraining maximum flow kinetic energy map

                    grass.run_command('r.out.gdal', input='_hflowii', output=pf+'_results/'+pf+'_aimec/depth/'+jfill+'.asc',
                        format='AAIGrid', overwrite=True) #exporting maximum flow height raster to ascii
                    grass.run_command('r.out.gdal', input='_tflowii', output=pf+'_results/'+pf+'_aimec/pressure/'+jfill+'.asc',
                        format='AAIGrid', overwrite=True) #exporting maximum flow kinetic energy raster to ascii

                    corrasc(pf+'_results/'+pf+'_aimec/depth/'+jfill)
                    corrasc(pf+'_results/'+pf+'_aimec/pressure/'+jfill)

                    flinei=flinet.replace('\n','') #preparing line for writing to input parameter file

                else:

                    grass.mapcalc('"%s_iis%s"=0' %(pf,jfill), overwrite=True, quiet=True) #impact indicator score map
                    grass.mapcalc('"%s_dis%s"=0' %(pf,jfill), overwrite=True, quiet=True) #deposit indicator score map
                    flinei='*'+flinet.replace('\n','') #preparing line for writing to input parameter file

                print>> fparami, flinei #writing line of temporary input parameter file to input parameter file

            fval.close() #closing validation file
            fparamt.close() #closing temporary input parameter file
            fparami.close() #closing input parameter file
            faimecp.close() #closing parameter input file for AIMEC


        # Finalizing impact indicator index maps

            for j in range(0,3):

                grass.mapcalc('"%s_iii%s"=float("%s_iii%s")/float(%s)'
                    %(pf, mstring[j],pf, mstring[j],str(nsuccess)), overwrite=True, quiet=True)
                
                grass.run_command('r.out.gdal', input=pf+'_iii'+mstring[j], output=ascpath+pf+'_iii'+mstring[j]+'.asc',
                    format='AAIGrid', overwrite=True) #exporting impact indicator index map to ascii
                corrasc(ascpath+pf+'_iii'+mstring[j])


        # Finalizing deposit indicator index maps

            grass.mapcalc('"%s_dii"=float("%s_dii")/float(%s)'
                %(pf, pf, str(nsuccess)), overwrite=True, quiet=True)
                
            grass.run_command('r.out.gdal', input=pf+'_dii', output=ascpath+pf+'_dii.asc',
                format='AAIGrid', overwrite=True) #exporting deposit indicator index map to ascii
            corrasc(ascpath+pf+'_dii')


        # Finalizing multi-core processing

            stop_batch = time.time() #storing time (stop of multi-core processing)
            comptime_batch = stop_batch - start_batch #storing computational time for batch processing in seconds

            timefile=open(filepath+pf+'_time.txt', 'w')
            timefile.write(str(comptime_batch)) #writing computational time for batch processing to file
            timefile.close()


        # Creating control point information file

            if ctrlpoints: #if control points are defined:

                fctrlpts=open(pf+'_results/'+pf+'_files/'+pf+'_ctrlpoints.txt', 'w') #opening control points information file for writing
                fctrlpts.write('Id\tx\ty\tiii\n') #writing header to file

                for ictrl in range(0,lctrlpts): #loop over all control points:

                    ctrlstring=grass.read_command('r.what', map=pf+'_iii_hflow', separator='\t', coordinates=xctrl[ictrl]+','+yctrl[ictrl],
                        overwrite=True, quiet=True) #reading impact indicator index at control point
                    ctrlstring=ctrlstring.replace('\t\t','\t').replace('\n','')

                    fctrlpts.write('C'+str(ictrl+1)+'\t'+ctrlstring+'\n')
                        #writing point number and coordinates to file

                fctrlpts.close() #closing file


    # Writing documentation file for flags and options

        documentation=open(pf+'_results/'+pf+'_files/'+pf+'_documentation.txt', 'w') #opening documentation file for writing

        if eflag: print>> documentation, 'Model execution\tTRUE'
        else: print>> documentation, 'Model execution\tFALSE'
        if kflag: print>> documentation, 'Keeping GRASS rasters of results\tTRUE'
        else: print>> documentation, 'Keeping GRASS rasters of results\tFALSE'
        if mflag: print>> documentation, 'Multiple model runs\tTRUE'
        else: print>> documentation, 'Multiple model runs\tFALSE'
        if vflag: print>> documentation, 'Validation and visualization\tTRUE'
        else: print>> documentation, 'Validation and visualization\tFALSE'

        print>> documentation, 'Prefix\t%s' %pf

        print>> documentation, 'Pixel size\t%s' %cellsize
        print>> documentation, 'Northern boundary\t%s' %rnorth
        print>> documentation, 'Southern boundary\t%s' %rsouth
        print>> documentation, 'Western boundary\t%s' %rwest
        print>> documentation, 'Eastern boundary\t%s' %reast

        print>> documentation, 'Model\t%s' %model
        print>> documentation, 'Topography\t%s' %topo
        print>> documentation, 'Flow height conversion\t%s' %funcctrl[0]
        print>> documentation, 'Diffusion control\t%s' %funcctrl[1]
        print>> documentation, 'Conservation of volume\t%s' %funcctrl[2]
        print>> documentation, 'Surface control\t%s' %funcctrl[3]
        print>> documentation, 'Entrainment and deposition\t%s' %funcctrl[4]
        print>> documentation, 'Stopping\t%s' %funcctrl[5]

        if cores: print>> documentation, 'Number of cores\t%s' %cores
        else: print>> documentation, 'Number of cores\tFALSE'
        if mflag: print>> documentation, 'Sampling strategy\t%s' %sampling
        else: print>> documentation, 'Sampling strategy\tFALSE'

        if impactarea: print>> documentation, 'Observed impact area\t%s' %impactarea
        else: print>> documentation, 'Observed impact area\tFALSE'
        if hdeposit: print>> documentation, 'Height of observed deposit\t%s' %hdeposit
        else: print>> documentation, 'Height of observed deposit\tFALSE'
        if topo=='2': print>> documentation, 'Area of interest\t%s' %aoi
        print>> documentation, 'Elevation\t%s' %elevation

        if hrelease: print>> documentation, 'Release height\t%s' %hrelease
        else: print>> documentation, 'Release height\tFALSE'
        if model=='2':
            if hreleases: print>> documentation, 'Solid release height\t%s' %hreleases
            else: print>> documentation, 'Solid release height\tFALSE'
            if hreleasef: print>> documentation, 'Fluid release height\t%s' %hreleasef
            else: print>> documentation, 'Fluid release height\tFALSE'
            if rhreleases: print>> documentation, 'Fraction of solids within release height\t%s' %rhreleases
            else: print>> documentation, 'Fraction of solids within release height\tFALSE'
        if mflag and vhrelease: print>> documentation, 'Variation of release height\t%s' %vhrelease
        else: print>> documentation, 'Variation of release height\tFALSE'
        print>> documentation, 'Use of release mass\t%i' %releasemass

        if hentrmax: print>> documentation, 'Maximum entrainable height\t%s' %hentrmax
        else: print>> documentation, 'Maximum entrainable height\tFALSE'
        if model=='2':
            if hentrmaxs: print>> documentation, 'Maximum entrainable solid height\t%s' %hentrmaxs
            else: print>> documentation, 'Maximum entrainable solid height\tFALSE'
            if hentrmaxf: print>> documentation, 'Maximum entrainable fluid height\t%s' %hentrmaxs
            else: print>> documentation, 'Maximum entrainable fluid height\tFALSE'
            if rhentrmaxs: print>> documentation, 'Fraction of solids within entrainable height\t%s' %rhentrmaxs
            else: print>> documentation, 'Fraction of solids within entrainable height\tFALSE'
        if mflag and vhentrmax: print>> documentation, 'Variation of entrainable height\t%s' %vhentrmax
        else: print>> documentation, 'Variation of entrainable height\tFALSE'

        if funcctrl[4]=='1' and centr: print>> documentation, 'Entrainment coefficient map\t%s' %centr
        else: print>> documentation, 'Entrainment coefficient map\tFALSE'
        if delta: print>> documentation, 'Basal friction angle map\t%s' %delta
        else: print>> documentation, 'Basal friction angle map\tFALSE'

        if hydrograph: print>> documentation, 'Hydrograph\t%s' %hydrograph
        else: print>> documentation, 'Hydrograph\tFALSE'
        if hydrocoords:
            print>> documentation, 'Hydrograph profile coordinates\t%s' %hydrocoords
            print>> documentation, 'Number of input hydrographs\t%i' %hyd_nin
            print>> documentation, 'Number of output hydrographs\t%i' %hyd_nout
        else: print>> documentation, 'Hydrograph profile coordinates\tFALSE'

        print>> documentation, 'Flow parameters\t%s' %flowparam
        if baseparam: print>> documentation, 'Basal surface parameters\t%s' %baseparam
        else: print>> documentation, 'Basal surface parameters\tFALSE'

        print>> documentation, 'Time interval for output\t%s' %tint
        print>> documentation, 'Time at which to stop simulation\t%s' %tstop
        print>> documentation, 'CFL criterion\t%s' %cfl[0]
        print>> documentation, 'Minimum time step length\t%s' %cfl[1]

        print>> documentation, 'Minimum flow height for simulation\t%s' %imparamc
        print>> documentation, 'Minimum flow height for display\t%s' %imparam[0]
        print>> documentation, 'Minimum flow kinetic energy for display\t%s' %imparam[1]
        print>> documentation, 'Minimum flow pressure for display\t%s' %imparam[2]

        if ctrlpoints: print>> documentation, 'Coordinates of control points\t%s' %ctrlpoints
        else: print>> documentation, 'Coordinates of control points\tFALSE'
        if profile: print>> documentation, 'Profile coordinates\t%s' %profile
        else: print>> documentation, 'Profile coordinates\tFALSE'
        if phexagg: print>> documentation, 'Vertical exaggeration factor for profile display\t%s' %phexagg
        else: print>> documentation, 'Vertical exaggeration factor for profile display\tFALSE'

        if not mflag: print>> documentation, 'Number of output time steps\t%i' %ntimesteps
        else: print>> documentation, 'Number of output time steps\tFALSE'
        if mflag: print>> documentation, 'Success of simulation\t%i' %nsuccess
        else: print>> documentation, 'Success of simulation\t%i' %csuccess

        documentation.close() #closing documentation file

    if vflag: #validation and visualization mode:

        print
        print "3. VALIDATION AND VISUALIZATION"
        print


    # Reading documentation file for flags and options

        if not eflag: #if mode only includes visualization:

            documentation=open(pf+'_results/'+pf+'_files/'+pf+'_documentation.txt', 'r') #opening documentation file
            
            documentation.readline()
            documentation.readline()
            kflag='TRUE'
            mflag=documentation.readline().replace('Multiple model runs\t', '').replace('\n', '')
            if mflag=='FALSE': mflag=None
            documentation.readline()

            pf=documentation.readline().replace('Prefix\t', '').replace('\n', '')

            cellsize=documentation.readline().replace('Pixel size\t', '').replace('\n', '')
            rnorth=documentation.readline().replace('Northern boundary\t', '').replace('\n', '')
            rsouth=documentation.readline().replace('Southern boundary\t', '').replace('\n', '')
            rwest=documentation.readline().replace('Western boundary\t', '').replace('\n', '')
            reast=documentation.readline().replace('Eastern boundary\t', '').replace('\n', '')

            model=documentation.readline().replace('Model\t', '').replace('\n', '')
            topo=documentation.readline().replace('Topography\t', '').replace('\n', '')
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()
            documentation.readline()

            documentation.readline()
            documentation.readline()

            impactarea=documentation.readline().replace('Observed impact area\t', '').replace('\n', '')
            if impactarea=='FALSE': impactarea=None
            hdeposit=documentation.readline().replace('Height of observed deposit\t', '').replace('\n', '')
            if hdeposit=='FALSE': hdeposit=None
            if topo=='2': documentation.readline()
            elevation=documentation.readline().replace('Elevation\t', '').replace('\n', '')

            if model=='1': hrelease=documentation.readline().replace('Release height\t', '').replace('\n', '')
            else: documentation.readline() 
            if model=='2':
                hreleases=documentation.readline().replace('Solid release height\t', '').replace('\n', '')
                hreleasef=documentation.readline().replace('Fluid release height\t', '').replace('\n', '')
                documentation.readline()
            documentation.readline()
            releasemass=int(documentation.readline().replace('Use of release mass\t', '').replace('\n', ''))

            if model=='1': hentrmax=documentation.readline().replace('Maximum entrainable height\t', '').replace('\n', '')
            else: documentation.readline()
            if model=='2':
                hentrmaxs=documentation.readline().replace('Maximum entrainable solid height\t', '').replace('\n', '')
                hentrmaxf=documentation.readline().replace('Maximum entrainable fluid height\t', '').replace('\n', '')
                documentation.readline()
            documentation.readline()

            documentation.readline()
            documentation.readline()

            hydrograph=documentation.readline().replace('Hydrograph\t', '').replace('\n', '')
            if hydrograph=='FALSE': hydrograph=None
            hydrocoords=documentation.readline().replace('Hydrograph profile coordinates\t', '').replace('\n', '')
            if not hydrocoords=='FALSE':
                hyd_nin=int(documentation.readline().replace('Number of input hydrographs\t', '').replace('\n', ''))
                hyd_nout=int(documentation.readline().replace('Number of output hydrographs\t', '').replace('\n', ''))
            else:
                hydrocoords=None
                hyd_nin=0
                hyd_nout=0

            documentation.readline()
            documentation.readline()

            tint=documentation.readline().replace('Time interval for output\t', '').replace('\n', '')
            tstop=documentation.readline().replace('Time at which to stop simulation\t', '').replace('\n', '')
            documentation.readline()
            documentation.readline()
            documentation.readline()
            imparam=[documentation.readline().replace('Minimum flow height for display\t', '').replace('\n', '')]
            imparam.append(documentation.readline().replace('Minimum flow kinetic energy for display\t', '').replace('\n', ''))
            imparam.append(documentation.readline().replace('Minimum flow pressure for display\t', '').replace('\n', ''))

            ctrlpoints=documentation.readline().replace('Coordinates of control points\t', '').replace('\n', '')
            if ctrlpoints=='FALSE': ctrlpoints=None
            profile=documentation.readline().replace('Profile coordinates\t', '').replace('\n', '')
            if profile=='FALSE': profile=None
            phexagg=documentation.readline().replace('Vertical exaggeration factor for profile display\t', '').replace('\n', '')

            ntimesteps=documentation.readline().replace('Number of output time steps\t', '').replace('\n', '')
            success=int(documentation.readline().replace('Success of simulation\t', '').replace('\n', ''))
            if not ntimesteps=='FALSE': ntimesteps=int(ntimesteps)           

            documentation.close() #closing documentation file

            grass.run_command('g.region', flags='d') #setting default region
            grass.run_command('g.region', n=rnorth, s=rsouth, w=rwest, e=reast) #updating region
            grass.run_command('g.region', res=cellsize) #updating pixel size

            if impactarea: #if observed impact area is defined:
                grass.run_command('g.copy', rast=(impactarea,'_impactarea'), overwrite=True, quiet=True) #temporary impact area map

                maximpactarea=grass.raster_info(impactarea)['max'] #maximum value of impact area
                if maximpactarea>0: impdef='1' #setting control for impact area map
                else: impdef='0'
            else: impdef='0' #if observed impact area map is not defined, setting control to negative

            if hdeposit: #if height of deposit map is defined:
                grass.run_command('g.copy', rast=(hdeposit,'_hdeposit'), overwrite=True, quiet=True) #temporary height of deposit map

                maxhdeposit=grass.raster_info(hdeposit)['max'] #maximum value of height of deposit
                if maxhdeposit>0: depdef='1' #setting control for deposit map
                else: depdef='0'
            else: depdef='0' #if height of deposit map is not defined, setting control to negative

        else: #success of simulation:
            if mflag: success=nsuccess
            else: success=csuccess


    # Profile plots

        if profile and not mflag and success>0:

            maxhflow=grass.raster_info(pf+'_hflow_max')['max'] #maximum flow height

            for j in [1,2,4]: #loop over all parameters to be displayed as bar plots:

                if model=='1':
                    maxsflow1=grass.raster_info(pf+mstring[j]+'_max')['max'] #maximum flow parameter
                    maxsflow2=0.0
                elif model=='2':
                    maxsflow1=grass.raster_info(pf+mstring[j]+'s_max')['max'] #maximum solid flow parameter
                    maxsflow2=grass.raster_info(pf+mstring[j]+'f_max')['max'] #maximum fluid flow parameter

                d_hdisp=max(maxsflow1,maxsflow2) #unit conversion factor for scaling of plot:
                mconv=math.log10(d_hdisp)
                if mconv<3: mconv2='1'
                elif mconv<6: mconv2='0.001'
                elif mconv<9: mconv2='0.000001'
                elif mconv<12: mconv2='0.000000001'
                elif mconv<15: mconv2='0.000000000001'
                else: mconv2='0.000000000000001'

                maxsflow1=maxsflow1*float(mconv2) #converting units of maxima for second y axis
                maxsflow2=maxsflow2*float(mconv2)

                pnames=[] #initializing list of map images
                pnamesc=[] #initializing list of compressed profile images
                pwidthc=400 #defining width of compressed profile images

                for step in range(0,ntimesteps+1): #loop over all time steps:

                    if step<10: fill='000'+str(step) #formatting time step string
                    elif step<100: fill='00'+str(step)
                    elif step<1000: fill='0'+str(step)
                    else: fill=str(step)

                    subprocess.call('Rscript %s/r.avaflow.profile.r %s %s %i %s %s %s %s %s %s %s %s %s %s %s --slave --quiet'
                        %(scriptpath, pf, model, step, cellsize, maxhflow, maxsflow1, maxsflow2, phexagg, fill, tint, tstop, depdef,
                        str(j), mconv2), shell=True) #creating profiles with R

                    pimg=Image.open(pf+'_results/'+pf+'_plots/'+pf+'_profiles_timesteps/'+pf+mstring[j]+fill+'.png')
                        #opening profile image
                    pheightc=int(float(pimg.size[1])*float(pwidthc/float(pimg.size[0]))) #height of compressed profile image
                    pimgc=pimg.resize((pwidthc,pheightc), Image.BILINEAR) #compressed profile image
                    pimgc.save(temppath+'/'+mstring[j]+'c'+fill+'.png') #saving compressed profile image

                    pnames.append(pf+'_results/'+pf+'_plots/'+pf+'_profiles_timesteps/'+pf+mstring[j]+fill+'.png')
                        #updating list of profile images
                    pnamesc.append(temppath+'/'+mstring[j]+'c'+fill+'.png') #updating list of profile images

                pimages=[Image.open(pname) for pname in pnames] #opening profile images
                panimname=pf+'_results/'+pf+'_plots/'+pf+mstring[j]+'_profile.gif' #name of animated gif
                writeGif(panimname, pimages, duration=0.2) #writing animated gif of profile images

                pimagesc=[Image.open(pnamec) for pnamec in pnamesc] #opening compressed profile images
                panimnamec=pf+'_results/'+pf+'_plots/'+pf+mstring[j]+'_profilec.gif' #name of compressed animated gif
                writeGif(panimnamec, pimagesc, duration=0.2) #writing animated gif of compressed profile images


    # Hydrograph plots

        if (hydrograph or hydrocoords) and not mflag and success>0:

            for ihyd in range(0, hyd_nin+hyd_nout): #loop over all hydrographs:

                subprocess.call('Rscript %s/r.avaflow.hydrograph.r %s %s %s %s %s --slave --quiet'
                    %(scriptpath, pf, model, str(ihyd+1), str(hyd_nin), str(tint)), shell=True) #creating hydrograph plot with R


    # Map plots

        if profile and success>0: #if profile is defined, writing coordinates to file:

            profcoord=map(str, profile.split(',')) #splitting profile coordinate string
            proflength=len(profcoord) #reading number of profile coordinates
            for profnum in xrange(0,proflength,2): #loop over all coordinates:
                if profnum==0:
                    profilex=profcoord[profnum] #extracting x coordinate
                    profiley=profcoord[profnum+1] #extracting y coordinate
                else:
                    profilex=profilex+'\t'+profcoord[profnum] #extracting x coordinate
                    profiley=profiley+'\t'+profcoord[profnum+1] #extracting y coordinate

            fmapprof=open(temppath+'/fmapprof.txt', 'w') #opening text file for writing
            print>> fmapprof, '%s\n%s' %(profilex, profiley) #writing profile string to text file
            fmapprof.close() #closing file
            profdef='1' #control for profile

        elif success>0: profdef='0'

        if ctrlpoints and success>0: ctrlpts=str(lctrlpts) #control for control points
        elif success>0: ctrlpts='0'

        if (hydrograph or hydrocoords) and success>0: hydr='1' #control for hydrographs
        elif success>0: hydr='0'

        if impdef=='1' and success>0: #if observed impact area map is defined:

            grass.mapcalc('"_limpactarea"=if("%s_impactarea"==1,1,null())' %pf, overwrite=True, quiet=True) #binary map of impact area
            grass.run_command('r.to.vect', input='_limpactarea', output='d_limpactarea',
                type='area', flags='v'+'s', quiet=True, overwrite=True) #line vector of observed impact area
            grass.run_command('v.out.ogr', input='d_limpactarea', output=temppath,
                type='boundary', flags='c', quiet=True, overwrite=True) #exporting line vector of observed impact area to shape file

        if depdef=='1' and success>0: #if height of observed deposit map is defined:

            grass.mapcalc('"_ldeposit"=if("%s_hdeposit">0,1,null())' %pf, overwrite=True, quiet=True) #binary map of observed deposit
            grass.run_command('r.to.vect', input='_ldeposit', output='d_ldeposit',
                type='area', flags='v'+'s', quiet=True, overwrite=True) #line vector of observed deposit
            grass.run_command('v.out.ogr', input='d_ldeposit', output=temppath,
                type='boundary', flags='c', quiet=True, overwrite=True) #exporting line vector of observed deposit to shape file

        releasemass1=0
        if releasemass==1 and success>0: #if (solid) release height map is defined:

            if model=='1': #name of (solid) release height map:
                if not mflag: hrlss=pf+'_hflow0000'
                else: hrlss=pf+'_hflow00001'
            elif model=='2':
                if not mflag: hrlss=pf+'_hflows0000'
                else: hrlss=pf+'_hflows00001'

            if grass.raster_info(hrlss)['max']>0:
                releasemass1=1
                grass.mapcalc('"_lrelease"=if("%s">0,1,null())' %hrlss, overwrite=True, quiet=True) #binary map of (solid) release height
                grass.run_command('r.to.vect', input='_lrelease', output='d_lrelease',
                    type='area', flags='v'+'s', quiet=True, overwrite=True) #line vector of (solid) release height
                grass.run_command('v.out.ogr', input='d_lrelease', output=temppath,
                    type='boundary', flags='c', quiet=True, overwrite=True) #exporting line vector of (solid) release height

        releasemass2=0
        if model=='2' and releasemass==1 and success>0: #if fluid release height map is defined:

            if not mflag: hrlsf=pf+'_hflowf0000' #name of fluid release height map:
            else: hrlsf=pf+'_hflowf00001'
            if grass.raster_info(hrlsf)['max']>0:
                releasemass2=1
                grass.mapcalc('"_lreleasef"=if("%s">0,1,null())' %hrlsf, overwrite=True, quiet=True) #binary map of fluid release height
                grass.run_command('r.to.vect', input='_lreleasef', output='d_lreleasef',
                    type='area', flags='v'+'s', quiet=True, overwrite=True) #line vector of fluid release height
                grass.run_command('v.out.ogr', input='d_lreleasef', output=temppath,
                    type='boundary', flags='c', quiet=True, overwrite=True) #exporting line vector of fluid release height


        if not mflag and success>0: #for single model run:

            if model=='1':
                maxvflow=grass.raster_info(pf+'_vflow_max')['max'] #maximum flow velocity

            elif model=='2':
                maxvflow1=grass.raster_info(pf+'_vflows_max')['max'] #maximum solid flow velocity
                maxvflow2=grass.raster_info(pf+'_vflowf_max')['max'] #maximum fluid flow velocity
                maxvflow=max(maxvflow1,maxvflow2)

            maxhentr=grass.raster_info(pf+'_basechange_fin')['max'] #maximum height of entrainment
            maxhdep=grass.raster_info(pf+'_basechange_fin')['min']*-1 #maximum height of deposition
            if maxhentr>0 or maxhdep>0: jmap=4 #testing whether maps of height of entrainment and deposition should be created
            else: jmap=3

            for j in range(0,jmap): #loop over all sets of maps to be created

                if j<3:
                    d_hdisp=grass.raster_info(pf+mstring[j]+'_max')['max'] #absolute maximum raster value
                    d_hdispn=0
                else:
                    d_hdisp=grass.raster_info(pf+mstring[j]+'_fin')['max']
                    d_hdispn=grass.raster_info(pf+mstring[j]+'_fin')['min']
                mconv=math.log10(max(d_hdisp,-d_hdispn)) #unit conversion and number of digits for legend according to maximum value:
                if mconv<3:
                    mconv2='1'
                    if mconv<0: mdig=3
                    elif mconv<1: mdig=2
                    elif mconv<2: mdig=1
                    else: mdig=0
                elif mconv<6:
                    mconv2='0.001'
                    if mconv<4: mdig=2
                    elif mconv<5: mdig=1
                    else: mdig=0
                elif mconv<9:
                    mconv2='0.000001'
                    if mconv<7: mdig=2
                    elif mconv<8: mdig=1
                    else: mdig=0
                elif mconv<12:
                    mconv2='0.000000001'
                    if mconv<10: mdig=2
                    elif mconv<11: mdig=1
                    else: mdig=0
                elif mconv<15:
                    mconv2='0.000000000001'
                    if mconv<13: mdig=2
                    elif mconv<14: mdig=1
                    else: mdig=0
                else:
                    mconv2='0.000000000000001'
                    if mconv<16: mdig=2
                    elif mconv<17: mdig=1
                    else: mdig=0
                d_hdisp=d_hdisp*float(mconv2)
 
                if j==3: #for entrainment and deposition map:
                    d_hdispn=d_hdispn*float(mconv2)
                    jimp=0 #index of impact parameter
                    d_htrsh5='0'
                    d_htrsh4=str(round(d_hdisp,mdig)) #breaks for raster maps
                    d_htrsh3=str(round(d_hdisp*10/20,mdig))
                    d_htrsh2=str(round(d_hdisp*6/20,mdig))
                    d_htrsh1=str(round(d_hdisp*3/20,mdig))
                    if d_hdisp==0: d_htrshm=str(round(0,mdig))
                    else: d_htrshm=str(min(float(imparam[jimp])*float(mconv2),float(d_htrsh1)*0.75))
                        #minimum value displayed

                    d_htrsh5n='0'
                    d_htrsh4n=str(round(d_hdispn,mdig))
                    d_htrsh3n=str(round(d_hdispn*10/20,mdig))
                    d_htrsh2n=str(round(d_hdispn*6/20,mdig))
                    d_htrsh1n=str(round(d_hdispn*3/20,mdig))
                    if d_hdispn==0: d_htrshmn=str(round(0,mdig))
                    else: d_htrshmn=str(max(-1*float(imparam[jimp])*float(mconv2),float(d_htrsh1n)*0.75))
                        #minimum value displayed

                else: #for all other maps:
                    jimp=j
                    d_htrsh5=str(round(d_hdisp,mdig)) #breaks for raster maps
                    d_htrsh4=str(round(d_hdisp*12/20,mdig))
                    d_htrsh3=str(round(d_hdisp*8/20,mdig))
                    d_htrsh2=str(round(d_hdisp*5/20,mdig))
                    d_htrsh1=str(round(d_hdisp*2/20,mdig))
                    if d_hdisp==0: d_htrshm=str(round(0,mdig))
                    else: d_htrshm=str(min(float(imparam[jimp])*float(mconv2),float(d_htrsh1)*0.75))
                        #minimum value displayed

                    d_htrsh5n='0'
                    d_htrsh4n='0'
                    d_htrsh3n='0'
                    d_htrsh2n='0'
                    d_htrsh1n='0'
                    d_htrshmn='0'

                mnames=[] #initializing list of map images
                mnamesc=[] #initializing list of compressed map images
                mwidthc=400 #defining width of compressed map images

                for i in range(0,ntimesteps+2): #loop over all time steps plus one for maps of maximum values:

                    if i<10: fill='000'+str(i) #formatting time step string
                    elif i<100: fill='00'+str(i)
                    elif i<1000: fill='0'+str(i)
                    else: fill=str(i)

                    if i==0: grass.mapcalc('"_elmod"="%s_elev"' %pf, overwrite=True, quiet=True) #elevation map
                    elif i==ntimesteps+1: grass.mapcalc('"_elmod"="%s_elev"+"%s_basechange_fin"' %(pf, pf), overwrite=True, quiet=True)
                    else: grass.mapcalc('"_elmod"="%s_elev"+"%s_basechange%s"' %(pf, pf, fill), overwrite=True, quiet=True)
                    grass.run_command('r.relief', input='_elmod', output='_hillshade', azimuth='315', quiet=True, overwrite=True) #hillshade
                    grass.run_command('r.out.gdal', input='_hillshade', output=temppath+'/hillshade.asc',format='AAIGrid',
                    overwrite=True) #export hillshade map to ascii

                    if i<=ntimesteps: #strings for names of maps:
                        mstringt=pf+mstring[j]+fill
                        if model=='2': mstrings=pf+mstring[j]+'s'+fill
                        if model=='2': mstringf=pf+mstring[j]+'f'+fill
                    elif j<3:
                        mstringt=pf+mstring[j]+'_max'
                        if model=='2': mstrings=pf+mstring[j]+'s_max'
                        if model=='2': mstringf=pf+mstring[j]+'f_max'
                    else:
                        mstringt=pf+mstring[j]+'_fin'
                        if model=='2': mstrings=pf+mstring[j]+'s_fin'
                        if model=='2': mstringf=pf+mstring[j]+'f_fin'

                    if model=='1':

                        if j==3: #for entrainment and deposition map:

                            grass.mapcalc('"_a2"=if("%s"*%s<%s,-4,if("%s"*%s<%s,-3,if("%s"*%s<%s,-2,if("%s"*%s<%s,-1,if("%s"*%s<=%s,0,\
                                if("%s"*%s<=%s,1,if("%s"*%s<=%s,2,if("%s"*%s<=%s,3,4))))))))'
                                %(mstringt, mconv2, d_htrsh3n, mstringt, mconv2, d_htrsh2n, mstringt, mconv2, d_htrsh1n,
                                mstringt, mconv2, d_htrshmn, mstringt, mconv2, d_htrshm, mstringt, mconv2, d_htrsh1, mstringt, mconv2,
                                d_htrsh2, mstringt, mconv2, d_htrsh3), overwrite=True, quiet=True) #reclassifying map

                            grass.mapcalc('"_a2c"="_a2"', overwrite=True, quiet=True) #raster map for contour creation

                            contstep=1 #contour interval
                            contmin=-4 #minimum contour
                            contmax=5 #maximum contour

                        else: #for all other maps:

                            grass.mapcalc('"_a2"=if("%s"*%s<%s,0,if("%s"*%s<%s,1,if("%s"*%s<%s,2,if("%s"*%s<%s,3,if("%s"*%s<%s,4,5)))))'
                                %(mstringt, mconv2, d_htrshm, mstringt, mconv2, d_htrsh1, mstringt, mconv2, d_htrsh2, mstringt, mconv2,
                                d_htrsh3, mstringt, mconv2, d_htrsh4), overwrite=True, quiet=True) #reclassifying map
                            grass.mapcalc('"_a2c"="_a2"', overwrite=True, quiet=True) #raster map for contour creation

                            contstep=1 #contour interval
                            contmin=1 #minimum contour
                            contmax=5 #maximum contour

                    elif model=='2': #create composite map for Pudasaini two-phase model:

                        if j==3: #for entrainment and deposition map:

                            grass.mapcalc('"_a21"="%s"*%s' %(mstringt, mconv2), overwrite=True, quiet=True)
                                #solid plus fluid components, unit conversion
                            grass.mapcalc('"_a22"=if("%s"==0,0,if("%s"==0,1,"%s"/("%s")))' %(mstrings, mstringf, mstrings, mstringt),
                                overwrite=True, quiet=True) #ratio of solid component
                            if i==ntimesteps+1: grass.mapcalc('"_a22"="_a22"*"_a21"/("%s"*%s+"%s"*%s)'
                                %(mstrings, mconv2, mstringf, mconv2), overwrite=True, quiet=True)
                                #correcting ratio of solid component for maximum

                            grass.mapcalc('"_a2"=if("_a21"<%s,-16,if("_a21"<%s,-12,if("_a21"<%s,-8,if("_a21"<%s,-4,if("_a21"<=%s,0,\
                                if("_a21"<=%s,1,if("_a21"<=%s,5,if("_a21"<=%s,9,13))))))))'
                                %(d_htrsh3n, d_htrsh2n, d_htrsh1n, d_htrshmn, d_htrshm, d_htrsh1, d_htrsh2, d_htrsh3),
                                overwrite=True, quiet=True) #reclass for magnitude
                            grass.mapcalc(
                                '"_a2"=if("_a2"==0,0,if("_a22"<0.25,"_a2",if("_a22"<0.5,"_a2"+1,if("_a22"<0.75,"_a2"+2,"_a2"+3))))',
                                overwrite=True, quiet=True) #reclass for ratio
                            grass.mapcalc('"_a2c"=if("_a2">0,"_a2"+3,if("_a2"==0,"_a2"+3,"_a2"-3))', overwrite=True, quiet=True)

                            contstep=4 #contour interval
                            contmin=-20 #minimum contour
                            contmax=20 #maximum contour

                        else: #for all other maps:

                            grass.mapcalc('"_a21"="%s"*%s' %(mstringt, mconv2), overwrite=True, quiet=True)
                                #solid plus fluid components, unit conversion
                            grass.mapcalc('"_a22"=if("%s"==0,0,if("%s"==0,1,"%s"/("%s")))' %(mstrings, mstringf, mstrings, mstringt),
                                overwrite=True, quiet=True) #ratio of solid component
                            if i==ntimesteps+1: grass.mapcalc('"_a22"="_a22"*"_a21"/("%s"*%s+"%s"*%s)'
                                %(mstrings, mconv2, mstringf, mconv2), overwrite=True, quiet=True)
                                #correcting ratio of solid component for maximum

                            grass.mapcalc('"_a2"=if("_a21"<=%s,0,if("_a21"<=%s,1,if("_a21"<=%s,6,if("_a21"<=%s,11,if("_a21"<=%s,16,21)))))'
                                %(d_htrshm, d_htrsh1, d_htrsh2, d_htrsh3, d_htrsh4), overwrite=True, quiet=True) #reclass for magnitude
                            grass.mapcalc('"_a2"=if("_a2"==0,0,if("_a22"<0.2,"_a2",if("_a22"<0.4,"_a2"+1,if("_a22"<0.6,"_a2"+2,\
                                if("_a22"<0.8,"_a2"+3,"_a2"+4)))))', overwrite=True, quiet=True) #reclass for ratio
                            grass.mapcalc('"_a2c"="_a2"+4', overwrite=True, quiet=True) #raster map for contour creation

                            contstep=5 #contour interval
                            contmin=0 #minimum contour
                            contmax=25 #maximum contour

                    grass.run_command('r.out.gdal', input='_a2', output=temppath+'/a2.asc', format='AAIGrid', 
                        overwrite=True) #export composite map to ascii raster

                    maxhstep=grass.raster_info('_a2')['max'] #maximum value of composite map
                    minhstep=grass.raster_info('_a2')['min'] #maximum value of composite map
                    if not maxhstep==0 or not minhstep==0:

                        grass.run_command('r.contour', input='_a2c', output='d_contours',
                            step=contstep, minlevel=contmin, maxlevel=contmax, quiet=True, overwrite=True)
                                #creating contour lines of magnitude
                        grass.run_command('v.out.ogr', flags='c', input='d_contours', output=temppath, type='line',
                            quiet=True, overwrite=True) #exporting contour lines to shape file

                    subprocess.call(
                    'Rscript %s/r.avaflow.map.r %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s'\
                        ' %s %s %s %s %s %s %s %s %s' %(scriptpath, temppath, pf, model, str(i), fill, contstep, d_htrsh5, d_htrsh4,
                        d_htrsh3, d_htrsh2, d_htrsh1, d_htrshm, d_htrsh5n, d_htrsh4n, d_htrsh3n, d_htrsh2n, d_htrsh1n, d_htrshmn,
                        rnorth, rsouth, rwest, reast, cellsize, tint, tstop, impdef, depdef, str(maxvflow), profdef, hydr,
                        str(releasemass1+2*releasemass2), str(hyd_nin), str(hyd_nout), mstring[j], mconv2, str(mdig), str(ntimesteps),
                        ctrlpts), shell=True) #creating maps with R

                    if i<=ntimesteps: #for all time steps (not maps of maximum value), prepare maps for animated gifs

                        mimg=Image.open(pf+'_results/'+pf+'_plots/'+pf+'_maps_timesteps/'+pf+mstring[j]+fill+'.png') #opening map image
                        mheightc=int(float(mimg.size[1])*float(mwidthc/float(mimg.size[0]))) #height of compressed map image
                        mimgc=mimg.resize((mwidthc,mheightc), Image.BILINEAR) #compressed map image
                        mimgc.save(temppath+'/'+mstring[j]+'c'+fill+'.png') #saving compressed map image

                        mnames.append(pf+'_results/'+pf+'_plots/'+pf+'_maps_timesteps/'+pf+mstring[j]+fill+'.png')
                            #updating list of map images
                        mnamesc.append(temppath+'/'+mstring[j]+'c'+fill+'.png') #updating list of compressed map images

                mimages=[Image.open(mname) for mname in mnames] #opening map images
                manimname=pf+'_results/'+pf+'_plots/'+pf+mstring[j]+'_map.gif' #name of animated gif
                writeGif(manimname, mimages, duration=0.2) #writing animated gif of map images

                mimagesc=[Image.open(mnamec) for mnamec in mnamesc] #opening compressed map images
                manimnamec=pf+'_results/'+pf+'_plots/'+pf+mstring[j]+'_mapc.gif' #name of compressed animated gif
                writeGif(manimnamec, mimagesc, duration=0.2) #writing animated gif of compressed map images

        elif success>0: #for multiple model runs:

            d_htrsh5='1.00' #breaks of impact indicator index
            d_htrsh4='0.80'
            d_htrsh3='0.60'
            d_htrsh2='0.40'
            d_htrsh1='0.20'
            d_htrshm='0.01'

            grass.run_command('r.relief', input=pf+'_elev', output='_hillshade', azimuth='315', quiet=True, overwrite=True) #hillshade
            grass.run_command('r.out.gdal', input='_hillshade', output=temppath+'/hillshade.asc', format='AAIGrid',
            overwrite=True) #export hillshade map to ascii

            for j in range(0,4): #loop over all sets of maps to be created

                if j<3: mstringx='_iii'+mstring[j]
                else: mstringx='_dii'

                grass.mapcalc(
                    '"_a2"=if("%s%s"<%s,0,if("%s%s"<%s,1,if("%s%s"<%s,2,if("%s%s"<%s,3,if("%s%s"<%s,4,5)))))'
                    %(pf, mstringx, d_htrshm, pf, mstringx, d_htrsh1, pf, mstringx, d_htrsh2, pf, mstringx, d_htrsh3,
                    pf, mstringx, d_htrsh4), overwrite=True, quiet=True) #reclassifying impact indicator index

                grass.run_command('r.out.gdal', input='_a2', output=temppath+'/a2.asc', format='AAIGrid', 
                    overwrite=True) #export reclassified map to ascii raster

                grass.mapcalc('"_a2c"="_a2"', overwrite=True, quiet=True) #raster map for contour creation

                contstep=1 #contour interval
                contmin=1 #minimum contour
                contmax=5 #maximum contour
                maxhstep=grass.raster_info('_a2')['max'] #maximum value of composite map
                if maxhstep>=contstep:

                    grass.run_command('r.contour', input='_a2c', output='d_contours',
                    step=contstep, minlevel=contmin, maxlevel=contmax, quiet=True, overwrite=True) #creating contour lines

                    grass.run_command('v.out.ogr', flags='c', input='d_contours', output=temppath, type='line',
                        quiet=True, overwrite=True) #exporting contour lines to shape file

                subprocess.call(
                'Rscript %s/r.avaflow.map.r %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s'\
                    ' %s %s %s %s %s %s %s %s'
                    %(scriptpath, temppath, pf, model, '1', 'iscore', contstep, d_htrsh5, d_htrsh4, d_htrsh3, d_htrsh2, d_htrsh1, d_htrshm,
                    '0', '0', '0', '0', '0', '0', rnorth, rsouth, rwest, reast, cellsize, tint, tstop, impdef, depdef, '0',
                    profdef, hydr, str(releasemass1+2*releasemass2), str(hyd_nin), str(hyd_nout), mstringx, '1', '2', '0', ctrlpts),
                    shell=True) #creating map with R


            # Validation through ROC plots

                if j==0 and impdef=='1':

                    grass.run_command('r.out.gdal', flags='c', input=impactarea, output=temppath+'/observed.asc',
                       format='AAIGrid', overwrite=True) #exporting map of observed impact area map to ascii raster
                    grass.run_command('r.out.gdal', flags='c', input=pf+mstringx, output=temppath+'/index.asc',
                        format='AAIGrid', overwrite=True) #exporting impact indicator index map to ascii raster

                    subprocess.call('Rscript %s/r.avaflow.roc.r %s %s %s %s %s --slave --quiet'
                        %(scriptpath, temppath, pf, '1', '1', '1'), shell=True)
                        #ROC plot relating iii to observation (without normalization)

                    subprocess.call('Rscript %s/r.avaflow.roc.r %s %s %s %s %s --slave --quiet'
                        %(scriptpath, temppath, pf, '2', '2', '1'), shell=True)
                        #ROC plot relating iii to observation (with normalization)

                if j==3 and depdef=='1':

                    grass.run_command('r.out.gdal', flags='c', input=hdeposit, output=temppath+'/observed.asc',
                        format='AAIGrid', overwrite=True) #exporting map of observed deposition height to ascii raster
                    grass.run_command('r.out.gdal', flags='c', input=pf+mstringx, output=temppath+'/index.asc',
                        format='AAIGrid', overwrite=True) #exporting deposition indicator map to ascii raster

                    subprocess.call('Rscript %s/r.avaflow.roc.r %s %s %s %s %s --slave --quiet'
                        %(scriptpath, temppath, pf, '3', '1', '2'), shell=True)
                        #ROC plot relating dii to observation (without normalization)

                    subprocess.call('Rscript %s/r.avaflow.roc.r %s %s %s %s %s --slave --quiet'
                        %(scriptpath, temppath, pf, '4', '2', '2'), shell=True)
                        #ROC plot relating dii to observation (with normalization)


# Cleaning file system and exiting

    os.system('rm -rf '+temppath) #removing temporary directory
    if eflag and not basechange: os.system('rm -rf '+ascpath+pf+'_basechange*') #removing obsolete ascii rasters

    grass.run_command('g.remove',flags='f', type='rast', pattern='_*', quiet=True) #removing temporary input and result raster maps
    grass.run_command('g.remove',flags='f', type='vect', pattern='d_*', quiet=True) #removing temporary vector maps

    if not kflag: grass.run_command('g.remove',flags='f', type='rast', pattern=pf+'_*', quiet=True) #removing result raster maps

    grass.run_command('g.region', flags='d') #resetting default region

    if eflag and not mflag:

        if csuccess==1:
            print
            print 'Completed successfully in %.1f seconds (net computing time excluding visualization).' %comptime
            print 'Please find the collected results in the directory %s_results.' %pf
            print

        elif csuccess==0:
            print
            print 'The simulation was interrupted due to numerical failure.'
            print 'Please find the collected results in the directory %s_results.' %pf
            print

    elif eflag and mflag:

        print
        print 'Completed in %.1f seconds (net computing time excluding visualization).' %comptime_batch
        print '%s out of %s simulations were successful.' %(str(nsuccess),str(nruns))
        print 'Please find the collected results in the directory %s_results.' %pf
        print

    sys.exit() #exit

if __name__=='__main__':
    options, flags=grass.parser()
    main()

